/*
 * filebrowser.c - Browser files 
 *
 * Copyright (C) 2012-2013  Davide Parise <bubini.mara5@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * All function share the same working path for simplify the send operation.
 * If want to know content of directory just set current path to the directory and scan it
 * whit changeDir() function and passing argument name=NULL
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "../constant.h"

#include "filebrowser.h"
#include "user_dirs.h"

char curr_path[MAX_PATH_LENGTH]; /* current path */
user_dirs_t *user_dirs;	/* user directories */


/*
 * initialize data and structure
*/
void init_filebrowser(){
	/* set path */
	if(user_dirs == NULL){/* if call two time do nothing */
		user_dirs = init_user_dirs();
		if(user_dirs == NULL){
			fprintf(stderr,"File Browser Error: user dirs is NULL\n");
			exit(-1);
			}
			print_user_dirs(user_dirs);
		}
	/* default path is home user */
	memset(curr_path,'\0',MAX_PATH_LENGTH);
	strncpy(curr_path,user_dirs->home,strlen(user_dirs->home));
}

/*
 * Free all resources 
*/
void free_filebrowser(){
	free_user_dirs(user_dirs);
}

/*************************************************/
/************ SET AND GET CURRENT PATH ***********/
/*************************************************/

/* 
 * set current path 
*/
void setCurrentPath(char *path){
	int len = strlen(path);
	if(len>MAX_PATH_LENGTH){
		fprintf(stderr,"Error set path:: path too large\n");
		return;
	}
	memset(curr_path,'\0',MAX_PATH_LENGTH);
	strncpy(curr_path,path,len);
}

void setCurrentPathHome(){
	setCurrentPath(user_dirs->home);
}
void setCurrentPathMusic(){
	setCurrentPath(user_dirs->music);
}
void setCurrentPathVideo(){
	setCurrentPath(user_dirs->video);
}
void setCurrentPathImages(){
	setCurrentPath(user_dirs->pict);
}
void setCurrentPathMedia(){
	setCurrentPath(user_dirs->home);	/* No implemented yet */
}
void setCurrentPathLBCS(){
	setCurrentPath(user_dirs->lbcs);
}
char *getCurrentPathHome(){
	if(user_dirs != NULL)
		return user_dirs->home;
	return NULL;
}
char *getCurrentPathMusic(){
	if(user_dirs != NULL)
		return user_dirs->music;
	return NULL;
}
char *getCurrentPathVideo(){
	if(user_dirs != NULL)
		return user_dirs->video;
	return NULL;

}
char *getCurrentPathMedia(){
	if(user_dirs != NULL)
		return user_dirs->home;		/* Not impemented yet */
	return NULL;
}
char *getCurrentPathLBCS(){
	if(user_dirs != NULL)
		return user_dirs->lbcs;
	return NULL;

}

char *getCurrentPath(){
	/* ?? or return only a copy of str ?? */
	return curr_path;
}

/************************************/
/************ SCAN DIR **************/
/************************************/

int select_file(const struct dirent *file){
//	if( (strcmp(file->d_name,".")==0) || (strcmp(file->d_name,"..")==0))
	if( (strcmp(file->d_name,".")==0))
		return 0;
	else
		return 1;
}

int alphasort2(const struct dirent **a, const struct dirent **b) {
#ifdef _DIRENT_HAVE_D_TYPE

	if((*a)->d_type == (*b)->d_type ){
		if( ((*a)->d_type == DT_DIR)){
			if(strcmp((*a)->d_name,"..") == 0)
				return 0;
			if(strcmp((*b)->d_name,"..") == 0)
				return 1;
		}
		//else
		return strcmp((*a)->d_name,(*b)->d_name);
	}
	else if((*a)->d_type == DT_DIR)	
		return 0;
	else return 1;
#else
	return strcmp((*a)->d_name,(*b)->d_name);
#endif
}

int simple_scandir(char *path,struct dirent ***result){
	return scandir(path,result,select_file,alphasort2);
}

/************************************/
/*********** CHANGE DIR *************/
/************************************/

int changeDirHome(struct dirent ***name_list){
	setCurrentPathHome();
	return simple_scandir(curr_path,name_list);
}
int changeDirMusic(struct dirent ***name_list){
	setCurrentPathMusic();
	return simple_scandir(curr_path,name_list);
}
int changeDirVideo(struct dirent ***name_list){
	setCurrentPathVideo();
	return simple_scandir(curr_path,name_list);
}
int changeDirImages(struct dirent ***name_list){
	setCurrentPathImages();
	return simple_scandir(curr_path,name_list);
}
int changeDir(char *name,struct dirent ***name_list){
	if(name != NULL){
		if(!strcmp(name,".")){
			printf("changeDir:: . \n");
		}
		else if(!strcmp(name,"..")){
			printf("changeDir:: .. \n");
			char *prev = rindex(curr_path,'/');
			memset(prev,0,strlen(prev));
		}
		else{
			printf("changeDir:: %s\n",name);
			strncat(curr_path,"/",1);
			strncat(curr_path,name,strlen(name));

		}
	}
	printf("filebrowse: curr_path = %s\n",curr_path);
	return simple_scandir(curr_path,name_list);
}

/************************************/
/*********** EXEC COMMAND ***********/
/************************************/

int computeRecursiveDir(char *dirName,int(*f)(char*)){
	int i,nfile;
	int result = 0;
	char tmp[255];
	struct dirent **d;
	
	
	nfile = simple_scandir(dirName,&d);
	for(i=0;i<nfile;i++){

		if((strcmp(d[i]->d_name,".")) && (strcmp(d[i]->d_name,".."))){

			memset(tmp,0,255);
			strncpy(tmp,dirName,strlen(dirName));
			strncat(tmp,"/",1);
			strncat(tmp,d[i]->d_name,strlen(d[i]->d_name));
			if(d[i]->d_type == DT_DIR)
				result += computeRecursiveDir(tmp,f);
			else
				result += f(tmp);
		}

		free(d[i]);
	}
	free(d);
	return result;

}


