/*
 * Davide Parise - Info about the local user directories extract from the file user_dirs.dirs
 * 
 * Copyright (C) 2012-2013  Davide Parise <bubini.mara5@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "../constant.h"
#include "user_dirs.h"

char *FILE_DIRS = {".lbcs/user-dirs.dirs"}; /* File name where are stored the info */

/* TAG of file $(FILE_DIRS) used by it */
const char *MUSIC_DIR = "XDG_MUSIC_DIR";
const char *VIDEO_DIR = "XDG_VIDEOS_DIR";
const char *PICTURES_DIR = "XDG_PICTURES_DIR";
const char *DOCUMENTS_DIR = "XDG_DOCUMENTS_DIR";
const char *DESKTOP_DIR = "XDG_DESKTOP_DIR";
/* other here ... */

/*
 * Print the default user dirs passing by argument
*/
void print_user_dirs(user_dirs_t *user_dir){
	printf("Users Dirs\n");
	if(user_dir==NULL){
		printf("empty\n");
		return;
	}

	printf("Home -> %s\n",user_dir->home);
	printf("Local -> %s\n",user_dir->lbcs);
	printf("Music -> %s\n",user_dir->music);
	printf("Video -> %s\n",user_dir->video);
	printf("Pictures -> %s\n",user_dir->pict);
	printf("Documents -> %s\n",user_dir->doc);

}

/*
 * load data whit format < ingnored string "/<$ENV>/ .. /<string_path>/ ..">
*/
void load_data_dir(char *str,char **dst){
	char result[MAX_PATH_LENGTH];
	char env_tmp[255];
	int result_count=0;
	int env_count=0;

	memset(result,'\0',MAX_PATH_LENGTH);
	int offset =0;
	for(;str[offset]!= '"';offset++);	/* find entry point */
		offset++;
	for(;str[offset]!= '"';offset++){
		if(offset>strlen(str)) return;	/* check */

		if(str[offset]=='$'){	/* substition */
			memset(env_tmp,'\0',255);
			env_count=0;
			offset++;	/* getenv whitout $ simbol */
			while((str[offset] != '/') && (str[offset] != '\n') && \
				(str[offset] != ' ') && (str[offset] != EOF) ) 
				env_tmp[env_count++] = str[offset++];
			char *tmp = getenv(env_tmp);
			if(tmp != NULL){
				strncpy(&result[result_count],tmp,strlen(tmp));
				result_count+=strlen(tmp);
			}
		}
		result[result_count] = str[offset];
		result_count++;
	}	/* while end  point */

	if(result[result_count-1] != '/'){
		result_count++;
	}
	result[result_count] = '\0'; 
	/* copy result */

	*dst = malloc(sizeof(char)*strlen(result)+1);
	if(dst == NULL){
		perror("load_data_dir: curr_dir");
		return;
	}
	memset(*dst,'\0',strlen(result)+1);
	strncpy(*dst,result,strlen(result));
}


/*
 * initialize structure that contain the commons user default path.
 * Read the file "~/.config/user-dirs.dirs" if exist and store data into structure 
 * otherwise return a structure whit the $HOME value for all entry. 
 * If an error occur a NULL value is returned
*/
user_dirs_t *init_user_dirs(){
	FILE *f;
	user_dirs_t *result;
	char tmp[MAX_PATH_LENGTH];
	int count=0;
	int len_home=0;
	char ch=0;

	/* initialize result */
	result = malloc(sizeof(user_dirs_t));
	if(result == NULL){
		perror("init_user_dir malloc user dirs");
		return NULL;
	}
	result->home=NULL;
	result->lbcs=NULL;
	result->music=NULL;
	result->video=NULL;
	result->pict=NULL;
	result->doc=NULL;

	/* add default directory $Home */
	load_data_dir("\"$HOME/\"",&result->home);
	if(result->home==NULL){
		fprintf(stderr,"Init User Dirs Error:Can not set default path Home\n \
			Possible Enviroment HOME not set");
		return NULL;
	}
	len_home=strlen(result->home);
	load_data_dir("\"$HOME/.lbcs\"",&result->lbcs);

	/* 
	 * read config file if exist 
	 * otherwise the default path home is set for all	
	 */
	memset(tmp,'\0',MAX_PATH_LENGTH);
	strncpy(tmp,result->home,strlen(result->home));
	strncat(tmp,FILE_DIRS,strlen(FILE_DIRS));
	printf("init_user_dirs reading config file %s\n",tmp);
	f = fopen(tmp,"r");
	if(f==NULL){
		perror("init_user_dirs:\n");
		printf("init_user_dirs:using default path\n");
		ch = EOF;/* not enter in while */
		//return result;
	}

	while(ch != EOF){
		ch = getc(f);
		if(ch == '#'){ /* comment line */
			/* do nothing read other line */
			for(;'\n'!= getc(f););
			
		}
		else if(ch!=EOF){
			
			if(ch != '\n'){
				tmp[count] = ch;
				count++;
			}
			else{
				count++;
				tmp[count] = '\0';	/* end of string */
				if(strncmp(tmp,MUSIC_DIR,strlen(MUSIC_DIR))==0){
					load_data_dir(tmp,&result->music);
				}
				else if(strncmp(tmp,VIDEO_DIR,strlen(VIDEO_DIR))==0){
					load_data_dir(tmp,&result->video);
				}
				else if(strncmp(tmp,PICTURES_DIR,strlen(PICTURES_DIR))==0){
					load_data_dir(tmp,&result->pict);
				}
				else if(strncmp(tmp,DOCUMENTS_DIR,strlen(DOCUMENTS_DIR))==0){
					load_data_dir(tmp,&result->doc);
				}

				memset(tmp,'\0',MAX_PATH_LENGTH);
				count = 0;
			}
		}
	}
	/* check if are not set and set it to the same value of home */
	if(result->music == NULL){
		result->music = malloc(sizeof(char)*(len_home+1));
		if(result->music==NULL){
			perror("Init User Dirs:\n");
			free(result);
			return NULL;
		}
		memset(result->music,'\0',len_home+1);
		strncpy(result->music,result->home,len_home);
	}
	if(result->video == NULL){
		result->video = malloc(sizeof(char)*(len_home+1));
		if(result->video==NULL){
			perror("Init User Dirs:\n");
			free(result);
			return NULL;
		}
		memset(result->video,'\0',len_home+1);
		strncpy(result->video,result->home,len_home);

	}
	if(result->pict == NULL){
		result->pict = malloc(sizeof(char)*(len_home+1));
		if(result->pict==NULL){
			perror("Init User Dirs:\n");
			free(result);
			return NULL;
		}
		memset(result->pict,'\0',len_home+1);
		strncpy(result->pict,result->home,len_home);

	}
	if(result->doc == NULL){
		result->doc = malloc(sizeof(char)*(len_home+1));
		if(result->doc==NULL){
			perror("Init User Dirs:\n");
			free(result);
			return NULL;
		}
		memset(result->doc,'\0',len_home+1);
		strncpy(result->doc,result->home,len_home);

	}
	fclose(f);

	return result;
}

void free_user_dirs(user_dirs_t *user_dirs){
	
	if(user_dirs == NULL)
		return;

	if(user_dirs->home != NULL)
		free(user_dirs->home);

	if(user_dirs->lbcs != NULL)
		free(user_dirs->lbcs);

	if(user_dirs->music != NULL)
		free(user_dirs->music);

	if(user_dirs->video != NULL)
		free(user_dirs->video);

	if(user_dirs->pict != NULL)
		free(user_dirs->pict);

	if(user_dirs->doc != NULL)
		free(user_dirs->doc);

	free(user_dirs);

}
