/* 
 * imageviewer.c - Lunch program gpicview and control it 
 *
 * Copyright (C) 2012-2013  Davide Parise <bubini.mara5@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/

#include "imageviewer.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/time.h>
#include <X11/keysym.h>

#include "../mouse/mouse.h"

#define VIEWER_OPEN	1
#define VIEWER_CLOSE	0
int viewer_state = VIEWER_CLOSE;	/* open = 1; closed = 0 */

long USECTOSEC = 1000000;	/* 1 sec converted into usec */
long LIMIT_USEC = 500000;	/* max usec passed for prevent double click */

int prevent_dclick;	/* prevent double click. otherwise can toggle fulscreen*/
struct timeval tv;	/* store current time */

/* util function for find double clivk */
set_prevent_dclick(){
	struct timeval ctv;
	
	prevent_dclick = 0;
	gettimeofday(&ctv,NULL);	/* curernt time */
	
	if((ctv.tv_sec - tv.tv_sec) == 0){
	  if((ctv.tv_usec - tv.tv_usec) < LIMIT_USEC){
	    prevent_dclick = 1;
	  }
	}
	else if((ctv.tv_sec - tv.tv_sec) == 1){
	  if( (USECTOSEC + ctv.tv_usec - tv.tv_usec) < LIMIT_USEC){
	    prevent_dclick = 1;
	  }
	}
	gettimeofday(&tv,NULL);
}

/* command viewer */

int viewer_start(){
	gettimeofday(&tv,NULL);
	mouse_start();	/* initialize mouse and xlib */
	return 0;
}
int viewer_end(){
	if(viewer_state == VIEWER_OPEN) /* quit an instance of gpicview if exist */
		viewer_quit();
	
	unset_windows_sendevent();
	mouse_end();
	return 0;
}
int viewer_quit(){
	pid_t pid;
	if(viewer_state == VIEWER_CLOSE)
		return 0;	/* do nothing */

	key_click(XK_q);
	wait(&pid);
	if(pid<0){
		perror("Viewer quitting\n");
		return -1;
	}

	viewer_state = VIEWER_CLOSE;
	return 0;
}

int viewer_openPhoto(const char *name){
	pid_t pid,child;
	
	if(viewer_state == VIEWER_OPEN) /* quit an instance of gpicview if exist */
		viewer_quit();

	set_window_sendevent_create();	/* set xlib to send event when a new window is created */
	
	pid = fork();
	if(pid < 0){
		perror("Viewer open photo\n");
		return -1;
	}
	else if(pid == 0){ /* child */
		int r = execlp("gpicview","gpicview",name,(char*)NULL);
		if(r == -1){
			perror("Viewer child open photo\n");
			return -1;
		}
		return 0;
	}
	else{	/* father */

		viewer_state = VIEWER_OPEN;
		if(wait_window_create("gpicview")==0){
		  fprintf(stderr,"Image Viewer Error: No Windows Created\n");
		  viewer_quit();
		}
		viewer_toggleFullscreen();	/* set full screen */
		return 0;
	}
}

/* command mode normal */
int viewer_nextPhoto(){
	key_click(XK_Right);
	return 0;

}
int viewer_prevPhoto(){
	key_click(XK_Left);
	return 0;

}
int viewer_rotatePhotoL(){
	key_click(XK_l);
	return 0;

}
int viewer_rotatePhotoR(){
	key_click(XK_r);
	return 0;

}
int viewer_toggleSlideshow(){
	key_click(XK_w);
	return 0;
}
int viewer_toggleFullscreen(){
	key_click(XK_F11);
	return 0;
}
/* command mode zoom */
int viewer_zoomIn(){
	key_click(XK_plus);
	return 0;

}
int viewer_zoomOut(){
	key_click(XK_minus);
	return 0;
}
int viewer_zoomDefault(){
	key_click(XK_f);
	return 0;
}
int viewer_mouveStart(){
  
	set_prevent_dclick();
	
	if(prevent_dclick==1)
	  return 1;

	mouse_moveInCenter();
	usleep(10);
	mouse_press(MOUSE_BUTTON_LEFT);
	return 0;

}
int viewer_mouveEnd(){
  	if(prevent_dclick==1)
	  return 1;

	mouse_release(MOUSE_BUTTON_LEFT);
	usleep(10);
	return 0;

}
int viewer_mouve(int x,int y){
  	if(prevent_dclick==1)
	  return 1;

	mouse_move(x,y);
	return 0;

}


