/*
 * lbcs.c - LinuxBlueControl server side library
 *
 * Copyright (C) 2012-2013  Davide Parise <bubini.mara5@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * All I/O socket call are blocking
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>

#include "constant.h"
#include "lbcs.h"

char *work_dir = "/.lbcs";	/* where is stored config file and other info */
char *file_user_dir = "/.config/user-dirs.dirs";	/* location of file contenent default user dirs */

/* Create, if not exist, a directory for use to store and load data */
int lbcs_init_files(){
	FILE *f2;
	DIR *d;
	pid_t pid;
	int child;
	char directory[MAX_PATH_LENGTH];	/* working directory */
	char *home;		/* home directory */
	char *dir_conf;		/* conf file where is stored default user directories */

	home = getenv("HOME");
	if(home==NULL){
		fprintf(stderr,"Init lbcs: Can not find environment varible HOME\n");
		return -1;
	}
	memset(directory,'\0',MAX_PATH_LENGTH);
	strncpy(directory,home,strlen(home));
	strncat(directory,work_dir,strlen(work_dir));
	printf("Init lib lbcs: set working directory %s\n",directory);

	d=opendir(directory);
	if( (d==NULL) && (errno == ENOENT)){
		/* file not exist must be create and initialize */
		printf("Creating directory\n");
		if(mkdir(directory,S_IRWXU|S_IRWXG|S_IXOTH)<0){
			perror("Init LBCS:create working directory\n");
			return -1;
		}
	}
	else if(d == NULL){	//&& errno != ENOENT
			perror("Init LBCS:working directory\n");
			return -1;
	}
	else{
		/* DO NOTHIG */

		printf("Init Lib Lbcs:Work directory exist\n");
		closedir(d);
		return 0;
	}
	/* Working directory created */

	/* Now creating config file for list of user dirs */

	dir_conf = malloc(sizeof(char)*(strlen(home)+strlen(file_user_dir)));
	memset(dir_conf,'\0',(strlen(home)+strlen(file_user_dir)));
	strncpy(dir_conf,home,strlen(home));
	strncat(dir_conf,file_user_dir,strlen(file_user_dir));

	printf("Init Lib Lbcs:Set config file %s\n",dir_conf);
	f2 = fopen(dir_conf,"r");
	if( (f2 == NULL) && (errno == ENOENT)){
		/* default file NOT exist CREATE EN EMPTY FILE */
		strcat(directory,"/user-dirs.dirs\0");
		printf("Init lib Lbcs: No config user file found\nCreating one empty %s\n",directory);
		printf("Init lib Lbcs WARNING: Disabled Navigation Path\nTry to install xdg-user-dirs package\n");
		pid = fork();
		if(pid <0){
			perror("Init Lib Lbcs:Create new file\n");
			return -1;
		}
		else if(pid == 0){
			int r= execlp("touch","touch",directory,(char*)NULL);
			if(r==-1){
				perror("Init Lib Lbcs:Can not copy file config directories\n");
				return -1;
			}
		}
		else{
			wait(&child);		
			if(child<0) return -1;
		}

	}
	else if(f2 == NULL){	/* an error occurs */
		perror("Init Lib Lbcs:config user dirs\n");
		return -1;
	}
	else{
		/* COPY FILE */
		fclose(f2);
		pid = fork();
		if(pid <0){
			perror("Init Lib Lbcs:Error copying file\n");
			return -1;
		}
		else if(pid==0){
			printf("Init Lib Lbcs:Copy config file...");

			int r= execlp("cp","cp",dir_conf,directory,(char*)NULL);
			if(r==-1){
				perror("\nInit Lib Lbcs:Can not copy file config directories\n");
				return -1;
			}
			exit(0);
		}
		else{
			wait(&child);
		}
		
	}

	printf("Init Lib Lbcs: Set Working Directory Finished\n");fflush(stdout);
	return 0;
}

/* initialize all data and structure for use library libLBCS */
int init_lbcs(){
	if(lbcs_init_files()<0){
		printf("init_lbcs Error occurs\n");
		 return -1;
	}

//	init_symple_print();	
	init_filebrowser();
	init_vlcrc();
	init_mouse();
	return 0;
}

int free_lbcs(){
//	free_symple_print();
	free_filebrowser();
	free_vlcrc();
	free_mouse();

	return 0;
}

