/*
 * mouse.c - Mouse and Keyboard functions - Provide communication whit xlib
 *
 * Copyright (C) 2012-2013  Davide Parise <bubini.mara5@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 * It is born for mouse functions whit xlib library
 * Now it implement more function how it is an extended class of xlib
 * For use it remeber to call before mouse_start() and when finish mouse_end()
 * 
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/XTest.h>

#include "mouse.h"


Display *display; /* current dispaly */
unsigned int dw,dh; /* dimension of the display (width and height )*/

/* initialize */
int init_mouse(){
	printf("Init Mouse emu\n");
	/* do nothing */
	return 0;
}
void free_mouse(){
}

int mouse_start(){
	int event_base,error_base,major_version,minor_version;

	display = XOpenDisplay(NULL);
	if(display==NULL){
		perror("Can not open display\n");
		return -1;
	}
	if(XTestQueryExtension(display,&event_base,&error_base,&major_version,&minor_version) == False){
		perror("Display not support extension\n");
		return -2;
	}

	dw = DisplayWidth(display,DefaultScreen(display));
	dh = DisplayHeight(display,DefaultScreen(display));

	return 0;
}

/* free resuorces */
void mouse_end(){
	XCloseDisplay(display);
}

/* mouve mouse */
void mouse_move(int x, int y){
	XWarpPointer(display, None,None, 0,0,0,0 ,x,y );
	XFlush(display);
}

/* mouve mouse in center of the screen */
void mouse_moveInCenter(){
	int deltaX,deltaY =0;
	XEvent event;

	int cx= dw/2;	/* x center of the display */
	int cy=dh/2;	/* y center of the */

	XQueryPointer(display, RootWindow(display, DefaultScreen(display)), 	/* display and window */
		&event.xbutton.root, &event.xbutton.window,	/* root and child window that the pointer is in */
		&event.xbutton.x_root,&event.xbutton.y_root,	/* coordinate to the root window' s origin */
		&event.xbutton.x,&event.xbutton.y,		/* coordinate relative to the specified window */
		&event.xbutton.state);				/* state of the modified state button */

	deltaX = cx - event.xbutton.x;
	deltaY = cy - event.xbutton.y;

	XWarpPointer(display, None,None, 0,0,0,0 ,deltaX,deltaY );
	XFlush(display);
}

/* press the mouse button */
void mouse_press(unsigned int button){
	XTestFakeButtonEvent(display,button,True,CurrentTime);
	XFlush(display);
}
/* release the mouse button */
void mouse_release(unsigned int button){
	XTestFakeButtonEvent(display,button,False,CurrentTime);
	XFlush(display);
}

/* click a mouse button */
void mouse_click(unsigned int button){
	XTestFakeButtonEvent(display,button,True,CurrentTime);
	XFlush(display);
	usleep(10);
	XTestFakeButtonEvent(display,button,False,CurrentTime);
	XFlush(display);
}

/* !! NOT YET !! keyboard */
void key_press(unsigned int keySym){
	KeyCode keycode = XKeysymToKeycode(display,keySym);

	XTestFakeKeyEvent(display,keycode,True,10);
	XFlush(display);

}
void key_relase(unsigned int keySym){
	KeyCode keycode = XKeysymToKeycode(display,keySym);

	XTestFakeKeyEvent(display,keycode,False,10);
	XFlush(display);
}
void key_click(unsigned int keySym){
	KeyCode keycode = XKeysymToKeycode(display,keySym);

	XTestFakeKeyEvent(display,keycode,True,10);
	XFlush(display);

	usleep(10);

	XTestFakeKeyEvent(display,keycode,False,10);
	XFlush(display);
}

/* 
 * set xlib to send event when a new window is created 
*/
void set_window_sendevent_create(){
	XSetWindowAttributes winAttr;
	Window w;

	if(display==NULL){	/* prevent */
		fprintf(stderr,"mouse:set win send event::display is NULL\n");
		return;
	}
	winAttr.event_mask = SubstructureNotifyMask;
	w = RootWindow(display,DefaultScreen(display));

	if(XChangeWindowAttributes(display,w,CWEventMask,&winAttr)<0){
		perror("Mouse Error:Can not change window attributes\n");
		return;
	}
}

/* 
 * Set empty to event mask of the root window 
*/
void unset_windows_sendevent(){
	XSetWindowAttributes winAttr;
	Window w;

	if(display==NULL){	/* prevent */
		fprintf(stderr,"mouse:unset win send event::display is NULL\n");
		return;
	}

	winAttr.event_mask = 0;	/* empty */
	w = RootWindow(display,DefaultScreen(display));

	if(XChangeWindowAttributes(display,w,CWEventMask,&winAttr)<0){
		perror("Mouse Error:Can not change window attributes\n");
		return;
	}

}
/*
 * Wait for new window create with name same as name passed by argument
 * @argument:
 *	name-> is the name of the window
 * @return:
 *	1 if it is created a new window with name equal to char* passed by argument
 * 	0 otherwise
*/

int wait_window_create(char *name){
	int try;
	XEvent event;
	char *name_win;

	if(display==NULL){
		fprintf(stderr,"wait_window_focusIn():Display is Null\n");
		return -1;
	}

	try=3;
	do{
		while(XPending(display)){
			memset(&event,0,sizeof(event));
			XNextEvent(display,&event);

			if(event.type == CreateNotify){
				XFetchName(display,event.xcreatewindow.window,&name_win);
				if(name_win!=NULL){
				if( strncmp(name,name_win,strlen(name)) == 0){
					XFree(name_win);
					/* clear queue */
					unset_windows_sendevent(); /* Not add into the queue */
					while(XPending(display)){
						XNextEvent(display,&event);
					}
					return 1;	/* OK */
				}
				}
				XFree(name_win);
			}
		}
		try--;
		sleep(1);	/* 3 seconds */
	}while(try != 0); 

	fprintf(stdout,"Mouse Warning:No window create with name %s\n",name);
	return 0;
}	


