/*
 * sympleLinkedList.c - A linked list implementation
 *
 * Copyright (C) 2012-2013  Davide Parise <bubini.mara5@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/

#include <stdio.h>
#include <stdlib.h> 

#include "sympleLinkedList.h"

/* item utility */
struct item_t *createItem(void *data){
	struct item_t *tmp = malloc(sizeof(struct item_t));
	if(tmp==NULL){
		perror("list_t createItem malloc error");
		return NULL;
	}
	tmp->data = data;
	tmp->t_next = NULL;
	tmp->t_prev = NULL;

	return tmp;
}

void *getDataItem(struct item_t *item){
	if(item == NULL)
		return NULL;
	return item->data;
}

/*
 * create an empty list
*/
struct list_t *createList(void){
	struct list_t *l;
	l=malloc(sizeof(struct list_t));
	if(l==NULL){
		perror("createList malloc error");
		return NULL;
	}
	l->head=NULL;
	l->tail=NULL;
	return l;
}

/*
 * test if the list is empty
*/
int isEmptyList(struct list_t *list){
	if(list == NULL) return 1;	/* list is not created , not exist (prevent error!) */
	else{
		if(list->head == NULL) return 1;
		else return 0;
	}
}

void clearList(struct list_t *list){
	struct item_t *iter,*curr;

	iter = getIterator(list);
	while(iter != NULL){
		curr = iter;
		iter = getNextItem(iter);
		free(curr);	
	}
	if(HEAD(list)!= NULL){
	//printf("DEBUG:: ERROR clearList head!= null %s\n",(char*)getDataItem(HEAD(list)));
	HEAD(list) = NULL;
	}
	if(TAIL(list)!= NULL){
	//printf("DEBUG:: ERROR clearList tail!= null %s\n",(char*)getDataItem(TAIL(list)));
	TAIL(list) = NULL;
	}

}
/*
 * clear all element in the list and free data
 * param:
 * @list
 *	the list
 * @f
 *	function used to free element if is null not free
*/
void clearListAndFree(struct list_t *list,void (*f)(void*)){
	void *data;
	struct item_t *iter,*curr;

	if(list == NULL){
		fprintf(stderr,"LIST: Can not clear and free a NULL list\n");
		return;	/* prevent error and ist not exist */
	}
	if(f==NULL)
		clearList(list);
	else{
		iter = getIterator(list);
		while(iter != NULL){
			data = getDataItem(iter);
			if(data != NULL)
				f(data);
			curr = iter;
			iter = getNextItem(iter);
			free(curr);	
		}
		HEAD(list) = NULL;
		TAIL(list) = NULL;
	}
}
/* delete the list created whit createList() command */
void deleteList(struct list_t *list,void f(void*)){
	clearListAndFree(list,f);
	free(list);
}
/*
* add data at the end of the list
*/
struct list_t *addItemList(struct list_t *list,void *data){
	struct item_t *item = createItem(data);
	if(item == NULL){
		return NULL;
	}
	if(list == NULL){
		perror("LIST: can not add element in NULL list\n");
		return;
	}
	if(isEmptyList(list)){
		list->head = item;
		list->tail = item;
	}
	else{		
		if(TAIL(list) == HEAD(list)){
			/* size == 1 */
			TAIL(list) = item;
			item->t_prev = HEAD(list);
			HEAD(list)->t_next = item;
		}
		else{
			TAIL(list)->t_next = item;
			item->t_prev = TAIL(list);
			TAIL(list) = item;
		}
	}
	return list;
}

/*
 * add item to head of the list
*/
struct list_t *addItemToHeadList(struct list_t *list,void *data){
	struct item_t *item = createItem(data);
	if(item == NULL){
		return NULL;
	}
	if(list == NULL){
		perror("LIST: can not add element in NULL list\n");
		return;
	}
	if(isEmptyList(list)){
		list->head = item;
		list->tail = item;
	}
	else{		
		if(TAIL(list) == HEAD(list)){
			/* size == 1 */
			HEAD(list) = item;
			item->t_next = TAIL(list);
			TAIL(list)->t_prev = item;
		}
		else{
			
			HEAD(list)->t_prev = item;
			item->t_next = HEAD(list);
			HEAD(list) = item;
		}
	}

	return list;
}
/* return the data stored at the head of the list */
void *getDataHeadList(struct list_t *list){
	if(isEmptyList(list))
		return NULL;
	return getDataItem(list->head);
}
void *getDataTailList(struct list_t *list){
	if(isEmptyList(list))
		return NULL;
	return getDataItem(TAIL(list));

}
void *getDataIndexList(struct list_t *list,int index){
	int len=0;
	struct item_t *iter;
	
	iter = list->head;
	for(;iter;iter=iter->t_next){
		if(len == index)
			return getDataItem(iter);
		len++;
	}
	return NULL;
	
}

/*
 * remove item in the list at index
 * return data of item removed
*/
void *removeDataIndexList(struct list_t *list,int index){
	int len=0;
	void *result;
	struct item_t *iter;
	
	iter = list->head;
	for(;iter;iter=iter->t_next){
		if(len == index){
			result = getDataItem(iter);
//			if(HEAD(list) == iter){
			if(iter->t_prev == NULL){ /* head index=0 */
				HEAD(list) = iter->t_next;
				iter->t_next = NULL;
				if(!isEmptyList(list))
					HEAD(list)->t_prev = NULL;
			}
//			if(TAIL(list) == iter)}
			else if(iter->t_next == NULL){ /* tail index=n */
				TAIL(list) = iter->t_prev;
				TAIL(list)->t_next = NULL;
				/* have sure more than one element */
			}
			else{	/* other */
				(iter->t_next)->t_prev = iter->t_prev;
				(iter->t_prev)->t_next = iter->t_next;
			}
			free(iter);
			return result;
		}
		len++;
	}
	return NULL;	
}
/*
 * number of element in the list
*/
int getLengthList(struct list_t *list){
	int len=0;
	struct item_t *iter;
	
	iter = list->head;
	for(;iter;iter=iter->t_next)
		len++;

	return len;
}

/*
 * Iterator functions
*/

struct item_t *getIterator(struct list_t *list){
	if(isEmptyList(list))
		return NULL;
	return HEAD(list);
}
struct item_t *getNextItem(struct item_t *iter){
	if(iter != NULL)
		return iter->t_next;
	return NULL;
}
struct item_t *getPrevItem(struct item_t *iter){
	if(iter != NULL)
		return iter->t_prev;
	return NULL;
}
int hasNextItem(struct item_t *iter){
	if(iter->t_next == NULL)
		return 1;
	else
		return 0;
}
int hasPrevItem(struct item_t *iter){
	if(iter->t_prev == NULL)
		return 1;
	else
		return 0;
}

/*
 * two utility functions for test the list when data item is a string
 * NOTE:is not part of the list's implementation
*/
void printListStringDataItem(struct list_t *list){
	char *tmp;
	struct item_t *iter;
	iter = list->head;
	for(;iter;iter=iter->t_next){
		tmp = (char*)getDataItem(iter);
		printf("%s\n",tmp);
	}
		
}
/*
 * print in the standar output and free element
*/
void printAndFreeStringDataItem(void *data){
	char *s = (char*)data;
	printf("Free elemnt %s\n",s);
	free(s);
}

