/*
 * sympleLinkedList.c - A linked list implementation
 *
 * Copyright (C) 2012-2013  Davide Parise <bubini.mara5@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/

#ifndef SYMPLELINKEDLIST_H
#define	SYMPLELINKEDLIST_H

#define TAIL(l) ((l)->tail)
#define HEAD(l) ((l)->head)

typedef struct item_t{
	void *data;
	struct item_t *t_next,*t_prev;
}item_list_t;

typedef struct list_t{
	struct item_t *head;
	struct item_t *tail;
}simply_list;

/*
 * create an empty list
*/
struct list_t *createList(void);

/*
 * test if the list is empty
*/
int isEmptyList(struct list_t *list);

/*
 * clear all element in the list and free data whit the function f passed by param
*/
void clearListAndFree(struct list_t *list,void f(void*));

/* delete completly the list created whit createList() command - clearListAndFree plus free list */
void deleteList(struct list_t *list,void f(void*));
/*
* add data at the end of the list
*/
struct list_t *addItemList(struct list_t *list,void *data);
/*
 * new head
*/
struct list_t *addItemToHeadList(struct list_t *list,void *data);

/*
 * return the data stored at the head of the list 
*/
void *getDataHeadList(struct list_t *list);
/*
 * return the data stored at the tail of the list 
*/
void *getDataTailList(struct list_t *list);
/*
*/
void *getDataIndexList(struct list_t *list,int index);

/*
 * remove item in the list at index
 * return data of item removed
*/
void *removeDataIndexList(struct list_t *list,int index);
/*
 * number of element in the list
*/
int getLengthList(struct list_t *list);


/* Iterator list functions */

struct item_t *getIterator(struct list_t *list);
struct item_t *getNextItem(struct item_t *iter);
struct item_t *getPrevItem(struct item_t *iter);
void *getDataItem(struct item_t *item);
int hasNextItem(struct item_t *iter);
int hasPrevItem(struct item_t *iter);

#endif

