/*
 * symplePrint.c - Print and Log functions that can be enable or disable at run-time
 *
 * Copyright (C) 2012-2013  Davide Parise <bubini.mara5@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * completed but not used for now
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <time.h>

#include "symplePrint.h"
#include "constant.h"

int print_mode;		/* store the print mode */
int log_mode;		/* store the log mode */
struct timeval tv;

FILE *filelog;

/* prototypes */
int null_fprintf(FILE *stream, const char *format, ...);	
int err_fprintf(FILE *stream, const char *format, ...);
int all_fprintf(FILE *stream,const char *format, ...);


/* 
 * Initialize simple mode of printing. Change it default mode at run-time by set_mode_* functions
 * By default is:
 *	print_mode: print only error to stderr
 *	log_mode: is disabled
 *
*/
void init_symple_print(){
	print_mode = PRINT_ERR;
	log_mode = LOG_OFF;
	
	set_mode_print(print_mode);
	set_mode_log(log_mode,NULL);
	/* print */
	/* log */
	print_printlog_mode();
}

/*
 * Enable or disable print finctions
*/
void set_mode_print(int enable_print){
	print_mode = enable_print;

	if(enable_print == PRINT_OFF)	/* print nothing */
		symple_print = &null_fprintf;
	else if(enable_print == PRINT_ERR)	/* print err */
		symple_print = &err_fprintf;
	else
		symple_print = &all_fprintf;	/* print all*/


}

/*
 * Enable or disable log
*/
void set_mode_log(int enable_log,char *path){
	char tmp[MAX_PATH_LENGTH];	/* is stored the path + name of the file log */
	char *name_filelog = "lbcs.log";

	log_mode = enable_log;

	if(enable_log == LOG_ON){	/* log enable */
		memset(tmp,'\0',MAX_PATH_LENGTH);

		if(path == NULL){
			fprintf(stderr,"symplePrint:Enable log fail\n");
			log_mode = LOG_OFF;
			return;
		}
		strncpy(tmp,path,strlen(path));
		strncat(tmp,name_filelog,strlen(name_filelog));

		filelog = fopen(tmp,"w+");
		if(filelog==NULL){
			perror("init print:Can not create log file\n");
			log_mode = LOG_OFF;
			return;
		}
		fprintf(filelog,"# Log file of LinuxBlueControl server side\n");
	}

}
/*
 * Free resuorces 
*/
void free_symple_print(){
	if(filelog != NULL)
		fclose(filelog);
}

/***************************************************/
/************ Print Functions **********************/
/***************************************************/


/* Not print */
int null_fprintf(FILE *stream, const char *format, ...){
	/* do nothing */

	/* log */
	if( (log_mode == LOG_ON) && (filelog != NULL) ){	/* log enabled */
		va_list args;
		gettimeofday(&tv,NULL);
		fprintf(filelog,"[%lu.%lu] ",tv.tv_sec,tv.tv_usec);
		fprintf(filelog,format,args);
	}
	return 0;
}

/* Print only stderr */
int err_fprintf(FILE *stream, const char *format, ...){
	/* print only err */
	if(stream==stderr){
		va_list args;
		fprintf(stream,format,args);
	}
	/* log */
	if( (log_mode == LOG_ON) && (filelog != NULL) ){	/* log enabled */
		va_list args;
		gettimeofday(&tv,NULL);
		fprintf(filelog,"[%lu.%lu] ",tv.tv_sec,tv.tv_usec);
		fprintf(filelog,format,args);
	}

	return 0;
}

/* Print all */
int all_fprintf(FILE *stream,const char *format, ...){
	va_list args;
	fprintf(stream,format,args);
	/* log */
	if( (log_mode == LOG_ON) && (filelog != NULL) ){	/* log enabled */
		va_list args;
		gettimeofday(&tv,NULL);
		fprintf(filelog,"[%lu.%lu] ",tv.tv_sec,tv.tv_usec);
		fprintf(filelog,format,args);
	}
}

/* Print current mode in the standard output */
void print_printlog_mode(){
	if(print_mode == 0)	/* print nothing */
		printf("init print:Enabled silent mode\n");
	else if(print_mode == 1)	/* print err */
		printf("inti print:Enabled err mode\n");
	else
		printf("init_print:Enabled all mode\n");

	if(log_mode == LOG_OFF)
		printf("init print:Log Disabled\n");
	else
		printf("init print:Log enabled\n");

}

