/*
 * filebrowser_handler.c - Handler the file browse request
 *
 * Copyright (C) 2012-2013  Davide Parise <bubini.mara5@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 * Thi is not used for now whit handler but use only the implemented functions!
 * The idea is to implemented new handler for apps that requires it
 *
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "h/msg.h"
#include "h/constant_msg.h"
#include "h/io.h"

#include "../lib/constant.h"
#include "../lib/file_browser/filebrowser.h"

#include "h/filebrowser_handler.h"

#define CMD_SETPATH	0x10
#define SETPATH_FILE	0x01
#define SETPATH_HOME	0x02
#define SETPATH_MUSIC	0x03
#define SETPATH_VIDEO	0x04
#define SETPATH_MEDIA	0x05

#define CMD_GETCURRPATH	0x11

#define RST_CHANGEDIR	0x12
#define CHANGEDIR_FILE	0x01
#define CHANGEDIR_HOME	0x02
#define CHANGEDIR_MUSIC	0x03
#define CHANGEDIR_VIDEO	0x04
#define CHANGEDIR_MEDIA	0x05

#define RST_COMPUTE_REC_DIR	0x13

msg_t ack_filebrowser[] = {ID_FILEBROWSER,FIELD_NOT_SET,TYPE_ACK,FIELD_NOT_SET,0x00,0x00,0x00,0x00};
msg_t err_filebrowser[] = {ID_FILEBROWSER,FIELD_NOT_SET,TYPE_ERROR,FIELD_NOT_SET,0x00,0x00,0x00,0x00};

/* 
 * sending dir content 
 * @param:
 *	dirname-> is the name of directory content in the current path of filebrowser_file.c
*/
void send_dir(char *dirname){
	register int i;
	int nfile;
	int size_name,size_string;
	uint8_t typeM,typeL;
	msg_t *msg;
	struct dirent **name_list;

	nfile = changeDir(dirname,&name_list);
	if(nfile<0){
		perror("CHANGEDIR while sending directory contenent\n");
		return;
	}

	/* SEND HOW MANY FILE */
	msg = createMsgI(ID_FILEBROWSER,FIELD_NOT_SET,TYPE_RESPONSE,RESPONSE_NMSG,nfile);
	if(msg==NULL){
		perror("filebrowser_handler malloc in send_dir for player\n");
		return;
	}
	if(send_msg(msg)<0){
		fprintf(stderr,"filebrowser_handler while sending directory can not send message\n");
		free(msg);
		return;
	}
	free(msg);

	/* WHITE ACK */
	if(read_msg(&msg)<0){
		fprintf(stderr,"filebrowser_handler while sending directory can not red message\n");
		return;
	}
	if(getMsgTypeM(msg,&typeM)<0){
		fprintf(stderr,"filebrowser_handler while sending directory can not parse message\n");
		free(msg);
		return;
	}
	if(typeM != TYPE_ACK){
		fprintf(stderr,"filebrowser_handler while sending directory can not received ack\nReceived 0x%.2x\n",typeM);
		free(msg);
		return;
	}
	free(msg);

	/* SEND ALL NAMES FILES */
	for(i=0;i<nfile;i++){
		size_name = strlen(name_list[i]->d_name);

		if(name_list[i]->d_type == DT_DIR)
			typeL = 0x01;
		else
			typeL = 0x02;
		
		msg = createMsg(ID_FILEBROWSER,FIELD_NOT_SET,TYPE_RESPONSE,typeL,size_name,name_list[i]->d_name);
		if(msg == NULL){
			perror("message in MUSICDIR\n");
			return;
		}

		if(send_msg(msg)<0){
			fprintf(stderr,"filebrowser_handler while sending directory can not send message\n");
			free(msg);
			while(i<nfile)
				free(name_list[i++]);
			free(name_list);			
			return;
		}
		free(msg);
		free(name_list[i]);
		/* WHITE ACK */
		if(read_msg(&msg)<0){
			fprintf(stderr,"filebrowser_handler while sending directory can not red message\n");
			i++;
			while(i<nfile)
				free(name_list[i++]);
			free(name_list);			
			return;
		}
		if(getMsgTypeM(msg,&typeM)<0){
			fprintf(stderr,"filebrowser_handler while sending directory can not parse message\n");
			freeMsg(msg);
			i++;
			while(i<nfile)
				free(name_list[i++]);
			free(name_list);			
			return;
		}
		if(typeM != TYPE_ACK){
			fprintf(stderr,"filebrowser_handler while sending directory can not received ack (%x)\n",typeM);
			freeMsg(msg);
			i++;
			while(i<nfile)
				free(name_list[i++]);
			free(name_list);			
			return;
		}
		free(msg);

	}
	free(name_list);
}


void filebrowser_handler(){
	uint8_t typeL,typeM,*data;
	msg_t *msg;
	int size;
	char tmp[MAX_FILENAME_LENGTH];

	init_filebrowser();

	for(;;){

	if(read_msg(&msg)<0){
		fprintf(stderr,"Error mouse_handler: can not read messages\n");
		freeMsg(msg);
		return;/* return to main handler or send error to client */
	}

	if(getMsgTypeM(msg,&typeM)<0){
		fprintf(stderr,"Error mouse_handler: %s\n",strmsgerr(getmsgerr()));
		freeMsg(msg);
		return;
	}
	if(getMsgTypeL(msg,&typeL)<0){
		fprintf(stderr,"Error mouse_handler: %s\n",strmsgerr(getmsgerr()));
		freeMsg(msg);
		return;
	}

	/* check message change state */
	if(typeM & TYPE_STATE_CHANGE){
		switch(typeL){
			case STATE_START:	/* never here only in the server switcher */
				fprintf(stdout,"FILEBROWSER HANDLER:: not normal !! change state start\n");
				freeMsg(msg);
				return;
			case STATE_STOP:
				printf("FILEBROWSER HANDLER:: command STOP\n");
				freeMsg(msg);
				if(send_msg(ack_filebrowser)<0)
					fprintf(stderr,"Error filebrowser_handler():can not send msg\n");
				return;
			case STATE_RESUME:	/* never here only in the server switcher */
				fprintf(stdout,"FILEBROWSER HANDLER:: not normal !! change state resume\n");
				freeMsg(msg);
				return;
			default:
				fprintf(stdout,"Change state:Unknow state 0x%.2x\n",typeL);
				freeMsg(msg);
				setMsgTypeL(err_filebrowser,ERROR_UNKNOW_COMMAND);
				if(send_msg(err_filebrowser)<0){
					fprintf(stderr,"Error mouse_handler():can not send msg\n");
				}
				return;
		};
	}
	/* check message command */
	else if(typeM & RST_CHANGEDIR){
		switch(typeL){
			case CHANGEDIR_FILE:
				fprintf(stdout,"FILEBROWSER HANDLER: change dir file\n");
				if(getMsgSize(msg,&size)<0){
					fprintf(stderr,"Player Handler error while parse msg (change dir)\n");
					freeMsg(msg);
					return;
				}
				memset(tmp,0,MAX_FILENAME_LENGTH);
				strncpy(tmp,(getPtrMsgData(msg)),size);
			
				send_dir(tmp);
				break;
			case CHANGEDIR_HOME:
				fprintf(stdout,"FILEBROWSER HANDLER: change dir home\n");
				setCurrentPathHome();
				send_dir(NULL);
				break;
			case CHANGEDIR_MUSIC:
				fprintf(stdout,"FILEBROWSER HANDLER: change dir music\n");
				setCurrentPathMusic();
				send_dir(NULL);
				break;
			case CHANGEDIR_VIDEO:
				fprintf(stdout,"FILEBROWSER HANDLER: change dir video\n");
				setCurrentPathVideo();
				send_dir(NULL);
				break;
			case CHANGEDIR_MEDIA:
				fprintf(stdout,"FILEBROWSER HANDLER: change dir media\n");
				setCurrentPathMedia();
				send_dir(NULL);
				break;
			default:
				fprintf(stderr,"Error FILEBROWSER HANDLER: change dir\n");
				if(send_msg(err_filebrowser)<0){
					fprintf(stderr,"Error FILEBROWSER HANDLER while sending error message\n");
					freeMsg(msg);
					return;
				}

		};
		freeMsg(msg);
	}
	else if(typeM & CMD_SETPATH){
		switch(typeL){
			case SETPATH_FILE:
				fprintf(stdout,"FILEBROWSER HANDLER: set path file\n");
				if(getMsgSize(msg,&size)<0){
					fprintf(stderr,"Player Handler error while parse msg (change dir)\n");
					freeMsg(msg);
					return;
				}
				memset(tmp,0,MAX_FILENAME_LENGTH);
				strncpy(tmp,(getPtrMsgData(msg)),size);

				setCurrentPath(tmp);
				break;
			case SETPATH_HOME:
				fprintf(stdout,"FILEBROWSER HANDLER: set path home\n");
				setCurrentPathHome();
				break;
			case SETPATH_MUSIC:
				fprintf(stdout,"FILEBROWSER HANDLER: set path music\n");
				setCurrentPathMusic();
				break;
			case SETPATH_VIDEO:
				fprintf(stdout,"FILEBROWSER HANDLER: set path video\n");
				setCurrentPathVideo();
				break;
			case SETPATH_MEDIA:
				fprintf(stdout,"FILEBROWSER HANDLER: set path media\n");
				setCurrentPathMedia();
				break;
			default:
				fprintf(stderr,"Error FILEBROWSER HANDLER: set path\n");
				setMsgTypeL(err_filebrowser,ERROR_UNKNOW_COMMAND);
				if(send_msg(err_filebrowser)<0){
					fprintf(stderr,"Error FILEBROWSER HANDLER while sending error message\n");
					freeMsg(msg);
					return;
				}

		};
		freeMsg(msg);
		if(send_msg(ack_filebrowser)<0){
			fprintf(stderr,"Error FILEBROWSER HANDLER while sending ack\n");
			return;
		}
	
	}
	else if(typeM & CMD_GETCURRPATH){
		fprintf(stdout,"FILEBROWSE HANDLER: get current path\n");
		freeMsg(msg);
		msg = createMsgC(ID_FILEBROWSER,FIELD_NOT_SET,TYPE_RESPONSE,RESPONSE_DATA,getCurrentPath());
		if(send_msg(msg)<0){
			fprintf(stderr,"Error FILEBROWSER HANDLER while sending current path\n");
			freeMsg(msg);
			return;
		}
		/* ??? read ack ??? */
		
	}//else if
}//FOR
}
