/*
 * imageviewer_handler.c - Handler request for view images. Use gpicview program
 *
 * Copyright (C) 2012-2013  Davide Parise <bubini.mara5@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "h/msg.h"
#include "h/constant_msg.h"
#include "h/io.h"

#include "../lib/constant.h"
#include "../lib/imageviewer/imageviewer.h"
#include "../lib/file_browser/filebrowser.h"

#include "h/imageviewer_handler.h"
#include "h/filebrowser_handler.h"

#define CMD_START	0x01
#define CMD_QUIT	0x02
#define CMD_OPEN_PHOTO	0x03
#define CMD_NEXT_PHOTO	0x04
#define CMD_PREV_PHOTO	0x05
#define CMD_ROTATE_L	0x06
#define CMD_ROTATE_R	0x07
#define CMD_ZOOM_IN	0x08
#define CMD_ZOMM_OUT	0x09
#define CMD_MOUVE_START	0x0A
#define CMD_MOUVE_END	0x0B
#define CMD_MOUVE	0x0C
#define CMD_TOGGLE_SLIDESHOW	0x0D
#define CMD_ZOOM_DEFAULT	0x0E


#define TYPE_RST_BROWSE	0x0F
#define RST_CHANGEDIR	0x41
#define RST_MUSICDIR	0x42
#define RST_HOMEDIR	0x43
#define RST_VIDEODIR	0x44
#define RST_IMAGEDIR	0x45
#define RST_MEDIADIR	0x46
#define RST_CURRPATH	0x47


msg_t ack_imageviewer[] = {ID_IMAGEVIEWER,FIELD_NOT_SET,TYPE_ACK, FIELD_NOT_SET,0x00,0x00,0x00,0x00};
msg_t err_imageviewer[] = {ID_IMAGEVIEWER,FIELD_NOT_SET,TYPE_ACK,  FIELD_NOT_SET,0x00,0x00,0x00,0x00};

/* entry point */
void imageviewer_handler(){
	uint8_t typeL,typeM,*data;
	char tmp[MAX_FILENAME_LENGTH];
	uint32_t keyCode;
	int size;
	msg_t *msg;
	int coordX,coordY;

	viewer_start();
	init_filebrowser();
	printf("********** IMAGEVIEWER HANDLER ****************\n");
	
for(;;){

	if(read_msg(&msg)<0){
		fprintf(stderr,"Error imageviewer_handler: can not read messages\n");
		return;/* return to main handler or send error to client */
	}

	if(getMsgTypeM(msg,&typeM)<0){
		fprintf(stderr,"Error imageviewer_handler: %s\n",strmsgerr(getmsgerr()));
		freeMsg(msg);
		return;
	}
	if(getMsgTypeL(msg,&typeL)<0){
		fprintf(stderr,"Error imageviewer_handler: %s\n",strmsgerr(getmsgerr()));
		freeMsg(msg);
		return;
	}

	/* check message change state */
	if(typeM & TYPE_STATE_CHANGE){
		switch(typeL){
			case STATE_START:	/* never here only in the server switcher */
				fprintf(stdout,"IMAGEVIEWER HANDLER:: not normal !! change state start\n");
				break;
			case STATE_STOP:
				printf("STOP IMAGEVIEWER HANDLER\n");
				freeMsg(msg);
				viewer_end();
				if(send_msg(ack_imageviewer)<0)
					fprintf(stderr,"Error imageviewer_handler():can not send msg\n");
				return;
			case STATE_RESUME:	/* never here only in the server switcher */
				fprintf(stdout,"IMAGEVIEWER HANDLER:: not normal !! change state resume\n");
				break;
			default:
				printf("Change state IMAGEVIEWER HANDLER:Unknow state 0x%.2x\n",typeL);
				if(send_msg(err_imageviewer)<0){
					fprintf(stderr,"Error imageviewer_handler():can not send msg\n");
				}
		};
	}
	/* browse the file system */
	else if(typeM == TYPE_RST_BROWSE){
		switch(typeL){
			/* this other case must be changed */
			case RST_MUSICDIR:
				setCurrentPathMusic();
				send_dir(NULL);
				break;
			case RST_HOMEDIR:
				setCurrentPathHome();
				send_dir(NULL);
				break;
			case RST_CHANGEDIR:
				if(getMsgSize(msg,&size)<0){
					fprintf(stderr,"Player Handler error while parse msg (change dir)\n");
					freeMsg(msg);
					return;
				}
				memset(tmp,0,MAX_FILENAME_LENGTH);
				strncpy(tmp,(getPtrMsgData(msg)),size);
				send_dir(tmp);				
				break;
			case RST_VIDEODIR:
				setCurrentPathVideo();
				send_dir(NULL);
				break;
			case RST_IMAGEDIR:
				setCurrentPathImages();
				send_dir(NULL);
				break;
			case RST_CURRPATH:
				fprintf(stdout,"Imageviewer: Send current path not implementated yet\n");
				break;
		};
	}
	/* check message command */
	else{
		switch(typeM){
			case CMD_START:
				viewer_start();
				break;
			case CMD_QUIT:
				viewer_quit();
				break;
			case CMD_OPEN_PHOTO:
				data = getPtrMsgData(msg);
				getMsgSize(msg,&size);
				memset(tmp,0,MAX_FILENAME_LENGTH);
				strncpy(tmp,getCurrentPath(),strlen(getCurrentPath()));
				strncat(tmp,"/",1);
				strncat(tmp,data,size);

				viewer_openPhoto(tmp);
				break;
			case CMD_NEXT_PHOTO:
				viewer_nextPhoto();
				break;
			case CMD_PREV_PHOTO:
				viewer_prevPhoto();
				break;
			case CMD_ROTATE_L:
				viewer_rotatePhotoL();
				break;
			case CMD_ROTATE_R:
				viewer_rotatePhotoR();
				break;
			case CMD_ZOOM_IN:
				viewer_zoomIn();
				break;
			case CMD_ZOMM_OUT:
				viewer_zoomOut();
				break;
			case CMD_MOUVE_START:
				viewer_mouveStart();
				break;
			case CMD_MOUVE_END:
				viewer_mouveEnd();
				break;
			case CMD_MOUVE:
				/* mouve cursor */
				if(getMsgSize(msg,&size)<0){
					fprintf(stderr,"Error mouse_handler: %s\n",strmsgerr(getmsgerr()));
					freeMsg(msg);
					return;
				}
				if(size != (sizeof(int)*2) ){
					fprintf(stderr,"Error mouse_handler: no msg data\n");
					freeMsg(msg);
					return;/* or request to resend */
				}
				data = getPtrMsgData(msg);
				coordX = getInt32(data);
				coordY = getInt32(data+4);
				mouse_move(coordX,coordY);

				viewer_mouve(coordX,coordY);
				break;
			case CMD_TOGGLE_SLIDESHOW:
				viewer_toggleSlideshow();
				break;
			case CMD_ZOOM_DEFAULT:
				viewer_zoomDefault();
				break;
			default:
				fprintf(stdout,"ImageViewer No Command\n");
		};
		freeMsg(msg);
		/* send ack */
		if(send_msg(ack_imageviewer)<0){
			fprintf(stderr,"Player Handler error while sending ack\n");
			return;
		}
		
	}
}/* for end */		
}

