/*
 * io.c - Provide function for read and write messages by unix socket
 *
 * Copyright (C) 2012-2013  Davide Parise <bubini.mara5@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * All I/O socket call are blocking
*/

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <errno.h>

#include "h/io.h"

int client;	/* client fd */

/* Set the client */
void set_client_io(int socket){
	client = socket;
	/* msg read send and other ... */
}


/* 
 * read message and store it in a buffer pointed by msg
 * blocking call while message is readed 
 * @param:
 *	msg: a pointer where a message readed is stored
 * @return:
 *	number of byte readed or -1 if error 
*/
int read_msg(msg_t **msg){
	int result;
	uint8_t buf[MAX_MESSAGE_SIZE];	/* header + data */

	memset(buf,0,MAX_MESSAGE_SIZE);
	result = read(client, buf, sizeof(buf));
	if( result < 0 ) {
		perror("read_msg() Can not read byte");
		return result;
	}

	if(buffToMsg(msg,buf,result)<0){
		fprintf(stderr,"Error while converting message\n");
		return -1;
	}

	return result;	/* | ok */
}

/*
 * The same as read_msg_old but 
 * read header and after read data bytes if any 
*/
int read_msg_new(msg_t **msg){
	int result;
	int nread;
	uint32_t size;
	uint8_t buf[MAX_MESSAGE_SIZE];	/* header + data */

	memset(buf,0,MAX_MESSAGE_SIZE);
	/* read header */
	result = 0;
	do{
	nread = read(client, &buf[result], MSG_HEADER_SIZE);
	if( nread < 0 ) {
		if(errno != ECONNRESET)
			perror("read_msg() Can not read byte");
		return nread;
	}
	result+=nread;
	}while(result < MSG_HEADER_SIZE);

	if(buffToMsg(msg,buf,result)<0){
		fprintf(stderr,"Error while converting message\n %s",strmsgerr(getmsgerr()));
		return -1;
	}

	/* read data if any */
	getMsgSize(*msg,&size);
	setMsgSize(*msg,0);	/* now only header */
	nread=0;
	result=0;
	while(size>0){
		nread = read(client, &buf[result], size);
		if( nread < 0 ) {
			perror("read_msg() Can not read byte");
			return nread;
		}
		result+=nread;
		size-=nread;
	}

	if(addMsgData(*msg,buf,result)<0){
		fprintf(stderr,"Error while converting message\n %s",strmsgerr(getmsgerr()));
		return -1;
	}


	return result;	/* | ok */
}

/* 
 * send message pointer by msg to client 
 * @param:
 *	msg: a pointer to msg to send
 * @return:
 *	number of byte sended or value <0 if error 
*/
int send_msg(msg_t *msg){
	int result;
	int length;
	int nsend;
	uint8_t *buf;

	if(getMsgLength(msg,&length)<0){
		fprintf(stderr,"Error send_msg(): %s",strmsgerr(getmsgerr()));
		return getmsgerr();
	}

	/* ADDING WHILE */
	//msgtobuf(msg,buf);	/* is the same type */
	buf=msg;
	while(length>0){
		if((nsend=send(client,buf,length,0))<0){
			perror("Error send_msg() Can not send msg");
			return nsend;
		}
		length-=nsend;
	}
	return result;
}

