/*
 * localDevice.c - Provide function for manage local device whit blueZ library
 *
 * Copyright (C) 2012-2013  Davide Parise <bubini.mara5@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/

#include <stdio.h>
#include <stdlib.h>

#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>

#include <bluetooth/rfcomm.h>

#include "h/localDevice.h"

/* print the info about the device */
void print_dev_info(struct hci_dev_info *dev_info){
	printf("dev id %d address %s name %s\n",dev_info->dev_id,batostr(&(dev_info->bdaddr)),dev_info->name);
}

/* do nothing */
void free_local(){
	if(local_dev!=NULL)
		free(local_dev);
}

/* Initialize adapter for communication */
int initLocalDevice(){
	int dd,dev_id;
	int result;

	printf("Initialize Local device\n");
	
	dev_id = hci_get_route(NULL);
	if(dev_id < 0){
		/* No Device */
		perror("InitLoaclDevice:No Device Found\n");
		return -3;
	}
	dd = hci_open_dev(dev_id);	/* device descriptor */
	if(!dd){
		perror("initLocalDevice can not open device");
		return (-1);
	}

	/* store data */
	local_dev = malloc(sizeof(struct localDevice));
	if(local_dev==NULL){
		perror("Local Device");
		return -2;
	}
	local_dev->id = dev_id;	/* id */
	if(hci_read_local_name(dd,255,local_dev->name,1000)<0){	/* locl name */
		perror("InitLocalDevice:Read local name");
		return (-1);
	}

	printf("Local Name is: %s and id %d\n",local_dev->name,local_dev->id);	/* info */
	hci_devinfo(dev_id,&(local_dev->info));
	print_dev_info(&(local_dev->info));

	hci_close_dev(dd);

	free_local();
	return 0;	
}

/*
 * in file hci_lib.h
 *
struct hci_request {
	uint16_t ogf;
	uint16_t ocf;
	int      event;
	void     *cparam;
	int      clen;
	void     *rparam;
	int      rlen;
};
 *
 */

/*
 * testing
*/
int test_request(){
	int sock;	/* socket for comunications */
	int dev_id;	/* identify device */
	int dd;	/* device descriptor */
	int result;	/* result of the test */
	uint8_t mode;
	uint8_t auth;	/* stored authorization */
	read_local_name_rp local_name;	/* store info of the local name */
	read_simple_pairing_mode_rp pairing_mode;	/* store info about pairing mode */
	struct sockaddr_rc addr = { 0 };	/* socket address */

	printf("Initialize request ...\n");
	struct hci_request req;	/* struct for request */
	req.ogf = OGF_HOST_CTL; /* host controller and baseband */
	req.ocf = OCF_READ_LOCAL_NAME;	/* read local name */
	req.event = 0;		/* not used think */
	req.cparam = NULL;	/* not used think */
	req.clen = 0;		/* not used think */
	req.rparam = &local_name;	/* store info */
	req.rlen = READ_LOCAL_NAME_RP_SIZE;	/* size of rparam */

	printf("Open device\n");
	dev_id = hci_get_route(NULL);	/* first device available */
	dd = hci_open_dev(dev_id);	/* device descriptor */
	if(!dd){
		perror("initLocalDevice can not open device");
		return (-1);
	}
	printf("Send request\n");
	result = hci_send_req(dd, &req, 10000);
	if(result < 0){
		perror("hci_send_request");
		hci_close_dev(dd);
		return -2;
	}
	printf("Result: name { %s } status { 0x%x }\n",local_name.name,local_name.status);


	req.ogf = OGF_HOST_CTL; /* host controller and baseband */
	req.ocf = OCF_READ_AUTH_ENABLE;	/* auth */
	req.event = 0;		/* not used think */
	req.cparam = NULL;	/* not used think */
	req.clen = 0;		/* not used think */
	req.rparam = &auth;	/* store info */
	req.rlen = 1;	/* size of rparam */

	result = hci_send_req(dd, &req, 10000);
	if(result < 0){
		perror("hci_send_request");
		hci_close_dev(dd);
		return -2;
	}
	if(auth==AUTH_DISABLED)
		printf("Auth is Disable:value %d\n",auth);
	else if(auth == AUTH_ENABLED)
		printf("Auth is Enable: value %d\n",auth);
	else
		printf("Auth is %d\n",auth);

	/* read encripty mode */
	req.ogf = OGF_HOST_CTL; /* host controller and baseband */
	req.ocf = OCF_READ_ENCRYPT_MODE;	/* auth */
	req.event = 0;		/* not used think */
	req.cparam = NULL;	/* not used think */
	req.clen = 0;		/* not used think */
	req.rparam = &auth;	/* store info */
	req.rlen = 1;	/* size of rparam */

	result = hci_send_req(dd, &req, 10000);
	if(result < 0){
		perror("hci_send_request");
		hci_close_dev(dd);
		return -2;
	}
	switch(auth){
		case ENCRYPT_DISABLED:
			printf("Encripty Disable\n");
			break;
		case ENCRYPT_P2P:
			printf("Encripty P2P\n");
			break;
		case ENCRYPT_BOTH:
			printf("Encripty Both\n");
			break;
		default:
			printf("Encripty Unknow\n");
	};
	/* read pairing mode */	
	req.ogf = OGF_HOST_CTL; /* host controller and baseband */
	req.ocf = OCF_READ_SIMPLE_PAIRING_MODE;	/* pairing mode */
	req.event = 0;		/* not used think */
	req.cparam = NULL;	/* not used think */
	req.clen = 0;		/* not used think */
	req.rparam = &pairing_mode;	/* store info */
	req.rlen = READ_SIMPLE_PAIRING_MODE_RP_SIZE;	/* size of rparam */

	result = hci_send_req(dd, &req, 10000);
	if(result < 0){
		perror("hci_send_request: pairing mode");
		hci_close_dev(dd);
		return -2;
	}
	printf("Pairing mode: status %d, mode %d\n",pairing_mode.status,pairing_mode.mode);

	hci_close_dev(dd);	/* close device */
	return 0;
}

