
/*
 * LinuxBlueControlS - Server side program based on bluetooth technology for control some applications and devices
 *	 by Android client program LinuxBlueControl
 * 
 *
 * Copyright (C) 2012-2013  Davide Parise <bubini.mara5@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <libudev.h>

#include <sys/types.h>
#include <sys/stat.h>

#include "h/localDevice.h"
#include "h/remoteDevice.h"
#include "h/rfcomm_util.h"

#include "h/io.h"
#include "h/switcher.h"

#include "h/mouse_handler.h"
#include "h/player_handler.h"
#include "h/filebrowser_handler.h"
#include "h/systemControll_handler.h"
#include "h/imageviewer_handler.h"

#include "../lib/lbcs.h"
#include "../lib/constant.h"

const char *arg_help = "--help";
const char *arg_version = "--version";
const char *arg_daemon = "--daemon";
const char *enable_log = "--enable-log";

int fdlog;	/* file descriptor of the log file*/
int client;	/* the bluetooth client */

/* print usage */
print_usage(char *name){
	printf("Usage: %s [OPTION]\n\n",name);
	printf("Options\n");
	printf("\t %s: show this help\n",arg_help);
	printf("\t %s: print version information and exit\n",arg_version);
	printf("\t %s: run daemon - Unstable mode\n",arg_daemon);
	printf("\n");
}

/* call when signal is sent */
void terminate_handler(int signum){
	if(signum==SIGINT || signum==SIGTERM || signum==SIGHUP){
		fprintf(stdout,"Received Signal HUP ...\n ");

		/* free sources */
		free_lbcs();		
		close_sdp_session();
		rfcomm_close(client);
		unregister_apps_service();

		/* close log file */
		close(fdlog);

		exit(0);
	}
}
/* 
 * daemonize if it is possible 
 *redirect output to the file log lbcs.log in the $HOME directory 
*/
int daemonize(){
        pid_t pid, sid;
	char *filelog = "lbcs.log";
	char *home;
	char tmp[MAX_PATH_LENGTH];


#ifdef _BSD_SOURCE
	if(daemon(1,1)==-1)	/* not changedir and not close std */
		return -1;
#elif  _XOPEN_SOURCE
	if(_XOPEN_SOURCE < 500){
		if(daemon(1,1)==-1)	/* not changedir and not close std */
			return -1;
	}
	else{
		fprintf(stderr,"Init Daemonize:daemon can not start\n");
        	return 0;	
	}
#else
	fprintf(stderr,"Init Daemonize:daemon can not start\n");
        return 0;
#endif

        /* Change the current working directory  and create file log */
	home = getenv("HOME");

	fprintf(stdout,"Runnig in daemon mode\nWARNING:Thi mode is unstable\n");
	fprintf(stdout,"File log %s is located in %s\n",filelog,home);
	fprintf(stdout,"Reports bugs by sending mail to: davide@linuxbluecontrol.com\n");

        if ((chdir(home)) < 0) {
		fprintf(stderr,"Init Daemonize:Can not change current working directory\n");
                exit(-1);
        }
	fdlog = open(filelog,O_RDWR|O_CREAT|O_CLOEXEC|O_TRUNC,S_IRUSR|S_IWUSR | S_IRGRP|S_IWGRP);
	if(fdlog<0){
		fprintf(stderr,"Init Daemonize:Can not create log file\n");
		exit(-1);
	}

	/* redirect standard input and output to log file */
	if(dup2(fdlog,STDOUT_FILENO)==-1){
		fprintf(stderr,"Init Daemonize:Can not duplicate stdout\n");
		exit(-1);
	}
	if(dup2(fdlog,STDERR_FILENO)==-1){
		fprintf(stderr,"Init Daemonize:Can not duplicate stderr\n");
		exit(-1);
	}

}


/* wait bluetooth hci event from udev */
int wait_hci(){
	int cond;
	int fd;
	int ret;


	fd_set fds;

	struct udev *udev;
	struct udev_device *dev;
   	struct udev_monitor *mon;

	const char *action;
	char *add_action="add";
	
	udev = udev_new();
	if (!udev) {
		printf("Can't create udev\n");
		exit(1);
	}

	mon = udev_monitor_new_from_netlink(udev, "udev");
	udev_monitor_filter_add_match_subsystem_devtype(mon, "bluetooth", NULL);
	udev_monitor_enable_receiving(mon);

	fd = udev_monitor_get_fd(mon);

	cond=1;
	while(cond){
		
	FD_ZERO(&fds);
	FD_SET(fd, &fds);
		
	ret = select(fd+1, &fds, NULL, NULL, NULL);
	
	if (ret > 0 && FD_ISSET(fd, &fds)) {
		dev = udev_monitor_receive_device(mon);
		if (dev) {
			action = udev_device_get_action(dev);
			fprintf(stdout,"Acion = %s\n",action);
			if(strncmp(action,add_action,strlen(add_action))==0){
				fprintf(stdout,"Udev added device\n");
				cond=0;
			}
			udev_device_unref(dev);
		}
		else {
			printf("No Device from receive_device(). An error occured.\n");
			udev_unref(udev);
			return -1;

		}					
	}
	else{
		fprintf(stderr,"wait_hci:Select problem\n");
		udev_unref(udev);
		return -1;
	}
	}
	udev_unref(udev);
	return 0;       
}

/* main  loop */
int main(int argc,char **argv){
	int result;
	char *lname; /* user logged in */
	struct passwd *pwd;

	fprintf(stdout,"%s version 1.0\n",argv[0]);
	if(argc > 2){
		print_usage(argv[0]);
		return 0;
	}

	if(argc == 2){
		if((strncmp(argv[1],arg_daemon,strlen(arg_daemon)) == 0 )){
			if(daemonize() == -1){
				perror("Deamon not start\n");
				exit(-1);
			}
			// else daemon continue
		}
		//else if ... other here
		else{
			print_usage(argv[0]);
			return 0;
		}
	}


	printf("********** Local Device *********\n");
	do{
		printf("\tInitLocalDevice... \n");
		result = initLocalDevice();
		if(result < 0){
			if(result==-3){/* no Device found (can plugged device?) */
				if(wait_hci()<0) return -1;
				sleep(1);
			}
			else{
				printf("InitLocalDevice:Found error %d\n",result);
				return -1;
			}
		}
	}while(result<0);


	printf("*********** SDP Register **********\n");
	register_service();

	printf("********** Init  *********\n");
	init_lbcs();	/* Initialize library */
	init_switcher();	/* initilize switcher */

	printf("********** Register Services *********\n");
	register_app_service(ID_EMUMOUSE,"Mouse Handler","emulate mouse",mouse_handler);
	register_app_service(ID_PLAYER,"Player Handler","Remote VLC Controller",player_handler);
	register_app_service(ID_FILEBROWSER,"File Browser","Browse your file stored in the pc",filebrowser_handler);
	register_app_service(ID_SYSTEM,"System Command","Send System Command as shutdown restart",system_handler);
	register_app_service(ID_IMAGEVIEWER,"Image Viewer","Can view your image",imageviewer_handler);

	/* for exit to main loop */
	signal(SIGINT,terminate_handler);
	signal(SIGTERM,terminate_handler);
	signal(SIGHUP,terminate_handler);

	

	while(1){	/* main loop */
		printf("***** Accept rfcomm connection *****\n");

		client = rfcomm_accept();	/* accept a connection */
		set_client_io(client);		/* set socket for read and write */

		start_server();		/* start the loop of the switcher - read msg and switch app */

		rfcomm_close(client);	/* close connection */
	

	}
	return 0;
}

