/*
 * mouse_handler.c - Handler mouse functions request
 *
 * Copyright (C) 2012-2013  Davide Parise <bubini.mara5@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/

#include <stdio.h>

#include "h/msg.h"
#include "h/constant_msg.h"
#include "../lib/mouse/mouse.h"

#include "h/mouse_handler.h"

/* emumouse command */
#define CMD_MOVE 	0x20
#define CMD_BUTTON	0x10

#define CMD_KEY		0x13
#define KEY_CLICK	0x01

#define BUTTON_LEFT 	0x04
#define BUTTON_RIGHT	0x08
#define BUTTON_MIDDLE	0x0C
#define BUTTON_PRESSED	0x02
#define BUTTON_RELEASED	0x01
#define BUTTON_CLICK	0x03

/* emumouse request */

char *const_button_to_str[] = {"unknow button","Right Button","Middle Button","Left Button"};

msg_t err_mouse[] =	{ID_EMUMOUSE,FIELD_NOT_SET,TYPE_ERROR,FIELD_NOT_SET,0x00,0x00,0x00,0x00};
msg_t ack_mouse[] = 	{ID_EMUMOUSE,FIELD_NOT_SET,TYPE_ACK,  FIELD_NOT_SET,0x00,0x00,0x00,0x00};

/******************************/
/******* MOUSE PARSER *********/
/******************************/

unsigned int getButtonMask(uint8_t mask){
	/* xxxxbbxx bb is interested byte */
	//int result = (int)((mask & 0x0C)>>2);
	/* must change constant must be equal to Xlib const */
	unsigned int result = 0;
	int button = (int) (mask & 0x0C);

	switch(button){
		case BUTTON_LEFT:
			result = MOUSE_BUTTON_LEFT;
			break;
		case BUTTON_RIGHT:
			result = MOUSE_BUTTON_RIGHT;
			break;
		case BUTTON_MIDDLE:
			result = MOUSE_BUTTON_MIDDLE;
			break;
	};

	return result;
}

int getEventMask(int mask){
	/* xxxxxxbb */
	int result = (int) (mask & 0x03);

	return result;
}
/* entry point */
void mouse_handler(){
	uint8_t typeL,typeM,*data;
	int coordX,coordY;
	uint32_t keyCode;
	int size;
	msg_t *msg;

	printf("******** MOUSE HANDLER *******\n");
	
	mouse_start();	/* initialize and start */

for(;;){	/* entering loop */

	if(read_msg(&msg)<0){
		fprintf(stderr,"Error mouse_handler: can not read messages\n");
		return;
	}

	if(getMsgTypeM(msg,&typeM)<0){
		fprintf(stderr,"Error mouse_handler: %s\n",strmsgerr(getmsgerr()));
		freeMsg(msg);
		return;
	}
	if(getMsgTypeL(msg,&typeL)<0){
		fprintf(stderr,"Error mouse_handler: %s\n",strmsgerr(getmsgerr()));
		freeMsg(msg);
		return;
	}

	/* check message change state */
	if(typeM & TYPE_STATE_CHANGE){
		switch(typeL){
			case STATE_START:	/* never here only in the server switcher */
				fprintf(stdout,"MOUSE HANDLER:: not normal !! change state start\n");
				break;
			case STATE_STOP:
				printf("MOUSE HANDLER:: command STOP\n");
				mouse_end();
				freeMsg(msg);
				if(send_msg(ack_mouse)<0)
					fprintf(stderr,"Error mouse_handler():can not send msg\n");
				return;
			case STATE_RESUME:	/* never here only in the server switcher */
				fprintf(stdout,"MOUSE HANDLER:: not normal !! change state resume\n");
				break;
			default:
				printf("Change state:Unknow state 0x%.2x\n",typeL);
				if(send_msg(err_mouse)<0){
					fprintf(stderr,"Error mouse_handler():can not send msg\n");
				}
		};
	}
	/* check message command */
	else if(typeM & CMD_MOVE){
		/* mouve cursor */
		if(getMsgSize(msg,&size)<0){
			fprintf(stderr,"Error mouse_handler: %s\n",strmsgerr(getmsgerr()));
			freeMsg(msg);
			return;
		}
		if(size != (sizeof(int)*2) ){
			fprintf(stderr,"Error mouse_handler: msg data (size data %d)\n",size);
			freeMsg(msg);
			return;/* or request to resend */
		}
		data = getPtrMsgData(msg);
		coordX = getInt32(data);
		coordY = getInt32(data+4);
		mouse_move(coordX,coordY);
	}
	else if(typeM & CMD_BUTTON){
		/* press button */
		unsigned int button = getButtonMask(typeM);
		if(button==0){
			fprintf(stderr,"Error getButtonMask unknow button/n");
			freeMsg(msg);
			return;
		}
		int evt = getEventMask(typeM);
		switch(evt){
			case BUTTON_PRESSED:
				printf("PRESS %s \n",const_button_to_str[button]);
				mouse_press(button);
				break;

			case BUTTON_RELEASED:
				printf("RELEASE %s \n",const_button_to_str[button]);
				mouse_release(button);
				break;
			case BUTTON_CLICK:
				printf("CLICK %s \n",const_button_to_str[button]);
				mouse_click(button);
				break;
			default:
				printf("Error:Unknow Command %s \n",const_button_to_str[button]);

		};
	}
	else if(typeM & CMD_KEY){
		if(getMsgSize(msg,&size)<0){
			fprintf(stderr,"Error mouse_handler: %s\n",strmsgerr(getmsgerr()));
			freeMsg(msg);
			return;
		}
		if(size != 4 ){
			fprintf(stderr,"Error mouse_handler: no msg data\n");
			freeMsg(msg);
			return;/* or request to resend */
		}
		keyCode = getInt32(getPtrMsgData(msg));
		switch(typeL){
			case KEY_CLICK:
			fprintf(stdout,"MOUSE HANDLER command key click keycode = %d\n",keyCode);
			key_click(keyCode);
			break;			
		};
	}
	else{
		printf("PARSE MSG MOUSE: command unknow\n");
	}


	freeMsg(msg);
	if(send_msg(ack_mouse)<0){
		fprintf(stderr,"Error mouse_handler():can not send msg\n");
		return;
	}
}//for
}

