/*
 * player_handler.c - Handler request for player. Use VLC program
 *
 * Copyright (C) 2012-2013  Davide Parise <bubini.mara5@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "h/msg.h"
#include "h/constant_msg.h"
#include "h/io.h"

/* my lib */
#include "../lib/util/sympleLinkedList.h"
#include "../lib/player/vlcrc.h"
#include "../lib/player/command.h"
#include "../lib/file_browser/filebrowser.h"

#include "h/player_handler.h"
#include "h/filebrowser_handler.h"


/* player */
#define CMD_START	0x00
#define CMD_QUIT	0x10
#define CMD_TOGGLEVLC	0x1F

#define CMD_PLAY	0x01
#define CMD_PAUSE	0x02
#define CMD_VOLUP	0x03
#define CMD_VOLDOWN	0x04
#define CMD_VOLMUTE	0x05
#define CMD_FULLSCREEN	0x06
#define CMD_ENQUEUEFILE	0x07
#define CMD_ENQUEUEDIR	0x08
#define CMD_ADDFILE	0x09
#define CMD_ADDDIR	0x0A
#define CMD_CLEAR	0x0B
#define CMD_NEXT	0x0C
#define CMD_PREV	0x0D
#define CMD_GOTOPLST	0x0E
#define CMD_STOP	0x13	/*0x0f reserved*/
#define CMD_SEEK	0x11
#define CMD_JUMP	0x12

#define RST_ERROR	0x40
#define RST_RESPONSE	0x41	/* TEMPORANY */
#define RST_ISPLAYING	0x47
#define RST_GETLENGTH	0X48
#define RST_GETTIME	0x49

#define RST_PLAYLIST	0x20	/* MSB */
#define RST_ISVLCSTART	0x21

/* for use utility browser */
#define TYPE_RST_BROWSE	0x0F
#define RST_CHANGEDIR	0x41
#define RST_MUSICDIR	0x42
#define RST_HOMEDIR	0x43
#define RST_VIDEODIR	0x44
#define RST_IMAGEDIR	0x45
#define RST_MEDIADIR	0x46
#define RST_CURRPATH	0x47


msg_t ack_player[] = 	{ID_PLAYER,FIELD_NOT_SET,TYPE_ACK, FIELD_NOT_SET,0x00,0x00,0x00,0x00};
msg_t err_player[] = 	{ID_PLAYER,FIELD_NOT_SET,TYPE_ACK,  FIELD_NOT_SET,0x00,0x00,0x00,0x00};

/******************************/
/******* PLAYER PARSER ********/
/******************************/

void send_playlist(){
	int nfile,i;
	msg_t *msg;
	uint8_t currType,typeM;
	char *name;
	struct list_t *playlist;
	struct item_t *iter;

	playlist = getPlaylist();
	nfile = getLengthList(playlist);

	/* SEND HOW MANY ITEM INTO PLAYLIST */
	msg = createMsgI(ID_PLAYER,FIELD_NOT_SET,TYPE_RESPONSE,RESPONSE_NMSG,nfile);
	if(msg==NULL){
		perror("malloc in send_dir for player\n");
		return;
	}
	if(send_msg(msg)<0){
		fprintf(stderr,"player_handler while sending playlist can not send message\n");
		free(msg);
		return;
	}
	free(msg);

	/* WHITE ACK */
	if(read_msg(&msg)<0){
		fprintf(stderr,"player_handler while sending playlist can not red message\n");
		return;
	}
	if(getMsgTypeM(msg,&typeM)<0){
		fprintf(stderr,"player_handler while sending playlist can not parse message\n");
		freeMsg(msg);
		return;
	}
	if(typeM != TYPE_ACK){
		fprintf(stderr,"player_handler while sending playlist can not received ack\n");
		freeMsg(msg);
		return;
	}
	free(msg);

	
	/* send files */
	iter = getIterator(getPlaylist());
	while(iter != NULL){

		/* SEND ITEM */
		name = rindex(getDataItem(iter),'/');
		name++;
		if(name == NULL){
			printf("Error while sending playlist item\n");
			return;
		}

		int size_name = strlen(name);

		msg = createMsg(ID_PLAYER,FIELD_NOT_SET,TYPE_RESPONSE,FIELD_NOT_SET,size_name,name);
		if(msg == NULL){
			perror("player handler while send playlist can not create message\n");
			return;
		}
		if(send_msg(msg)<0){
			fprintf(stderr,"player_handler while sending directory can not send message\n");
			freeMsg(msg);
			return;
		}
		free(msg);

		/* WHITE ACK */
		if(read_msg(&msg)<0){
			fprintf(stderr,"player_handler while sending playlist can not red message\n");
			return;
		}
		if(getMsgTypeM(msg,&typeM)<0){
			fprintf(stderr,"player_handler while sending playlist can not parse message\n");
			freeMsg(msg);
			return;
		}
		if(typeM != TYPE_ACK){
			fprintf(stderr,"player_handler while sending playlist can not received ack received %x\n",typeM);
//			freeMsg(msg);
			return;
		}
		free(msg);
	
		/* NEXT ITEM */
		iter = getNextItem(iter);
	}
}


void player_handler(){
	int size,tmp_int;
	uint8_t type,typeL;
	char tmp[255];
	char *str_rline;	/* malloc whit call. Remember to free!! */
	msg_t *msg;
	void (*func)(char*);
	char *param;

	fprintf(stdout,"****************** PLAYER HANDLER *******************\n");
for(;;){
	if(read_msg(&msg)<0){
		fprintf(stderr,"Player hanlder error while read message\n");
		return;
	}
	if(getMsgTypeM(msg,&type)<0){
		fprintf(stderr,"Player Handler errorn while parse message\n");
		freeMsg(msg);
		return;
	}

	if(getMsgTypeL(msg,&typeL)<0){
		fprintf(stderr,"Player Handler errorn while parse message\n");
		freeMsg(msg);
		return;
	}

	if(type & TYPE_STATE_CHANGE){
		switch(typeL){
			case STATE_START:
				fprintf(stdout,"PLAYER HANDLER:: not normal !! change state start\n");
				break;
			case STATE_STOP:	/* (android) activity no visible */
				printf("PLAYER HANDLER:: command STOP\n");
				if(send_msg(ack_player)<0){
					fprintf(stderr,"Error mouse_handler():can not send msg\n");
				}
				freeMsg(msg);
				return;
			case STATE_RESUME:
				fprintf(stdout,"PLAYER HANDLER:: not normal !! change state start\n");
				break;
			default:
				printf("Change state:Unknow state 0x%.2x\n",typeL);
				if(send_msg(err_player)<0){
					fprintf(stderr,"Error mouse_handler():can not send msg\n");
				}
				return;

		};
	}
	/* browse the file system */
	else if(type == TYPE_RST_BROWSE){
		switch(typeL){
			/* this other case must be changed */
			case RST_MUSICDIR:
				setCurrentPathMusic();
				send_dir(NULL);
				break;
			case RST_HOMEDIR:
				setCurrentPathHome();
				send_dir(NULL);
				break;
			case RST_CHANGEDIR:
				if(getMsgSize(msg,&size)<0){
					fprintf(stderr,"Player Handler error while parse msg (change dir)\n");
					freeMsg(msg);
					return;
				}
				memset(tmp,0,255);
				strncpy(tmp,(getPtrMsgData(msg)),size);
				send_dir(tmp);				
				break;
			case RST_VIDEODIR:
				setCurrentPathVideo();
				send_dir(NULL);
				break;
			case RST_IMAGEDIR:
				setCurrentPathImages();
				send_dir(NULL);
				break;
			case RST_CURRPATH:
				fprintf(stdout,"Not Implementated yet\n");
				break;
		};
	}
	/* playlist request */
	else if(type == RST_PLAYLIST){
		send_playlist();
	}
	else if(type == RST_ISVLCSTART){
		freeMsg(msg);
		msg = createMsgI(ID_PLAYER,FIELD_NOT_SET,TYPE_RESPONSE,FIELD_NOT_SET,getIsStartVlcrc());
		send_msg(msg);
	}
	else{
	/* parse command or request */
		switch(type){
			case CMD_START:
				if(start_vlcrc()<0){
					fprintf(stderr,"Player Handler Error:Can not start Vlc\n");
					send_msg(err_player);
					freeMsg(msg);
					return;					
				}
				
				break;
			case CMD_PLAY:
				play();
				break;
			case CMD_PAUSE:
				pause();
				break;
			case CMD_VOLUP:
				volup("1");
				break;
			case CMD_VOLDOWN:
				voldown("1");
				break;
			case CMD_VOLMUTE:
				volmute();
				break;
			case CMD_FULLSCREEN:
				fullscreen(NULL);
				break;
			case CMD_ENQUEUEFILE:
			case CMD_ENQUEUEDIR:
				if(getMsgSize(msg,&size)<0){
					fprintf(stderr,"Player Handler error while parse msg (change dir)\n");
					freeMsg(msg);
					return;
				}
				memset(tmp,'\0',255);
				strncpy(tmp,(getPtrMsgData(msg)),size);
				
				compute(tmp,VLCRC_CMD_ENQUEUE);
				break;
			case CMD_ADDFILE:
			case CMD_ADDDIR:
				if(getMsgSize(msg,&size)<0){
					fprintf(stderr,"Player Handler error while parse msg (change dir)\n");
					freeMsg(msg);
					return;
				}
				memset(tmp,'\0',255);
				strncpy(tmp,(getPtrMsgData(msg)),size);
	
				compute(tmp,VLCRC_CMD_ADD);		
				break;
			case CMD_CLEAR:
				clearPlst();
				break;
			case CMD_NEXT:
				next();
				break;
			case CMD_PREV:
				prev();
				break;
			case CMD_GOTOPLST:
				gotoPlstI(getInt32((getPtrMsgData(msg)) ) );/* TEMPORANY ADD CONTROLL */
				break;
			case CMD_STOP:
				stop();
				break;
			case CMD_QUIT:
				quit_vlcrc();
				break;
			case CMD_SEEK:
				//seekI(getInt32(&msg[2]));
				seek((getPtrMsgData(msg)));
				break;
	
			case CMD_JUMP:
				jumpI(getInt32((getPtrMsgData(msg))));
				break;
			case CMD_TOGGLEVLC:
				toggle_vlcrc();
				break;
			case RST_ISPLAYING:
				fprintf(stderr,"player command isPlaying\nUNSUPPORTED COMMAND\n");
				break;
			case RST_GETLENGTH:
				fprintf(stderr,"player command getLength\nUNSUPPORTED COMMAND\n");
				break;
			case RST_GETTIME:
				fprintf(stderr,"player command getTime\nUNSUPPORTED COMMAND\n");
				break;
			default:
				printf("player command unknow 0x%.2x\nRemove data and send error to client\n",type);
				/* ADDING SEND ERROR */
		};
		freeMsg(msg);
		if(send_msg(ack_player)<0){
			fprintf(stderr,"Player Handler error while sending ack\n");
			return;
		}
		//free(msg);

	}
}//for
}

