
/*
 * remote_device.c - Function for register the service whit sdp
 *
 * Copyright (C) 2012-2013  Davide Parise <bubini.mara5@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <termios.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <sys/poll.h>

#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>

#include <bluetooth/sdp.h>
#include <bluetooth/sdp_lib.h>

#include <bluetooth/rfcomm.h>
#include <bluetooth/l2cap.h>

#include "h/localDevice.h"
#include "h/remoteDevice.h"

#define MAX_RSP 255

struct proto_t{
	uuid_t uuid_proto;	/* uuid of protocol */
	int port;		/* port of protocol */

	struct proto_t *next;	/* next proto */
};

struct service_t{
	uuid_t uuid_service;		/* uuid of service */
	struct proto_t *proto;		/* list of protocol for use service */

	struct service_t *next;		/* next service */
}*service_dev;

struct conn_info_t{
	uint16_t handle;
	int port;
	char desc[255];
};

struct conn_info_t conn_info[255];

void print_conn_info(int n){
	int i;
	
	for(i=0;i<n;i++){
		printf("%d) %s handle 0x%x\nAvailable on port %d\n",i,conn_info[i].desc,conn_info[i].handle,conn_info[i].port);
	}
}

void print_result(){
	int i;
	if(remote_dev.num_dev>0){
		printf("Device found %d\n",remote_dev.num_dev);
		for(i=0;i<remote_dev.num_dev;i++)
			printf("%d) name %s addr %s\n",i,remote_dev.name[i],batostr(&(remote_dev.dev[i].bdaddr)));
	}
	else
		printf("No device found!\n");
}
int scan(){
	int result,i,dd;
	inquiry_info *tmp_dev;
	char remote_name[255];
	dd = hci_open_dev(local_dev->id);
	if(!dd){
		perror("initLocalDevice");
		return (-1);
	}

	tmp_dev = malloc(MAX_RSP*sizeof(inquiry_info));
	if(!tmp_dev){
		perror("malloc in scan");
		return (-1);
	}
	printf("Scanning...\n");
	/* inquiry */
	result = hci_inquiry(local_dev->id,8,MAX_RSP,NULL,&tmp_dev,IREQ_CACHE_FLUSH);
	if(result<0){
		perror("inquiry");
		exit(0);
	}
	else if(result==0){
		printf("No device found\n");
	}
	else{
		printf("Found %d device\n",result);
		remote_dev.num_dev = result;
		((remote_dev).name) = (char**)malloc(result*sizeof(char**));
		remote_dev.dev = (inquiry_info*)malloc(result*sizeof(inquiry_info*));
		for(i=0;i<result;i++){
			memset(remote_name,'\0',255);
			hci_read_remote_name(dd,&(tmp_dev[i].bdaddr),255,remote_name,2000);
			printf("%d) addr %s name %s\n",i,batostr(&(tmp_dev[i].bdaddr)),remote_name);

			remote_dev.name[i] = (char*)malloc(result*sizeof(char*));
			strncpy(remote_dev.name[i],remote_name,strlen(remote_name));
			remote_dev.dev[i] = tmp_dev[i];
		}
	}

	hci_close_dev(dd);
	free(tmp_dev);

	return result;
}

sdp_session_t *GLOBAL_SESSION;
sdp_record_t *rec;

/*
 * close spd session
*/
void close_sdp_session(){

	if (!rec) {
		printf("Service Record not found.\n");
		sdp_close(GLOBAL_SESSION);
		return;
	}

	if(sdp_record_unregister(GLOBAL_SESSION, rec)){
		printf("Failed to unregister service record: %s\n", strerror(errno));
		sdp_close(GLOBAL_SESSION);
		return;
	}
	printf("Close sdp connection\n");
	sdp_close(GLOBAL_SESSION);
}

/*
 * register a service for sdp
*/
void register_service(){
	/* structure */
	int err;
	char str_uuid[MAX_LEN_UUID_STR];
	/*
	*	I DON T KOWN :
	*	the service is converted into little or big endian ??
	*	if yes what is it the time that is convertet ??
	*	I ignore it??
	*
	*	uint32_t service[] = {0xb31aa8d6, 0x27bd11e0, 0x8c71001f,0x3c03056d};
	*
	*	or..
	*/ 
	uint8_t service[] = {0xb3, 0x1a, 0xa8,0xd6, 0x27, 0xbd, 0x11, 0xe0, 0x8c, 0x71, 0x00, 0x1f,0x3c, 0x03, 0x05,0x6d};

	uint8_t rfcomm_channel = 8;	/* temporany */
	const char *service_name = "LinuxBlueControl";
	const char *service_dsc = "Control some app of Linux whit Android";
	const char *service_prov = "Linux Bluetooth Control";
	uuid_t root_uuid,l2cap_uuid,rfcomm_uuid,svc_uuid;
	sdp_list_t *root_list=0,*l2cap_list=0,*rfcomm_list=0,*proto_list=0,*access_proto_list=0,*svc_list=0;
	sdp_data_t *channel=0,*psm=0;

	/* insert data into structure */


	printf("spd::add service to sdp server\n");

	/* service ID */
	rec = sdp_record_alloc();	/* allocate new record */
	sdp_uuid128_create(&svc_uuid,&service);	/* creat service class id */
	sdp_set_service_id(rec,svc_uuid);	/* set service class id */
	svc_list = sdp_list_append(0,&svc_uuid);
	sdp_set_service_classes(rec,svc_list);

sdp_uuid2strn(&svc_uuid,str_uuid,MAX_LEN_UUID_STR);

	/* service public browse */
	sdp_uuid16_create(&root_uuid,PUBLIC_BROWSE_GROUP);
	root_list = sdp_list_append(0,&root_uuid);
	sdp_set_browse_groups(rec,root_list);

	/* l2cap connection */
	sdp_uuid16_create(&l2cap_uuid,L2CAP_UUID);
	l2cap_list = sdp_list_append(0,&l2cap_uuid);

	/* rfcomm connection */
	sdp_uuid16_create(&rfcomm_uuid,RFCOMM_UUID);
		channel = sdp_data_alloc(SDP_UINT8,&rfcomm_channel);	/* create data channel */
	rfcomm_list = sdp_list_append(0,&rfcomm_uuid);	/* adding protocol */
	sdp_list_append(rfcomm_list,channel);		/* adding channel */

	/* adding rfcomm and l2cap to proto list */
	proto_list = sdp_list_append(0,l2cap_list);	
	sdp_list_append(proto_list,rfcomm_list);	

	/* proto list */
	access_proto_list = sdp_list_append(0,proto_list);	/* create list of protocol */
	sdp_set_access_protos(rec,access_proto_list);		/* adding protocol to the record */

	/* set other info */
	sdp_set_info_attr(rec,service_name,service_prov,service_dsc);

	/* register service */

	printf("sdp::Register service %s in the local sdp \n",str_uuid);
	/* connect to the local sdp server */
	GLOBAL_SESSION = sdp_connect(BDADDR_ANY,BDADDR_LOCAL,SDP_RETRY_IF_BUSY);
	err = sdp_record_register(GLOBAL_SESSION,rec,0);
	if(err<0){
		perror("FAIL::sdp registering record");
		exit(-1);
	}

	printf("sdp::remove data allocated for sdp server\n");
	/* remove data allocates*/
	sdp_data_free(channel);
	sdp_list_free(rfcomm_list,0);
	sdp_list_free(root_list,0);
	sdp_list_free(access_proto_list,0);
	
//	sdp_close(GLOBAL_SESSION);
}
void search_service_dev_id(int dev){
	int result,conn_num;
	sdp_session_t *session;
	sdp_list_t *lst_response;

	sdp_list_t *search;/*Each entry in the list is a UUID(DataTypeSDP_UUID16) of the service to be searched*/
	uuid_t rfccom_uuid;

	sdp_list_t *attr_range;		/* list o attribute but SDP_ATTR_REQ_RANGE */
	uint32_t range = 0x0000ffff;	/* 0x0000 to 0xFFFF gets all attributes */

	printf("Creating pdu...\n");
	sdp_uuid16_create(&rfccom_uuid, RFCOMM_UUID);	/* 128 in the example */
	search = sdp_list_append(0,&rfccom_uuid);	/* adding service to search rfccom*/
	attr_range = sdp_list_append(0,&range);		/* attribute range */

	if(local_dev ==NULL){
		printf("No device initialized\n");
		return;
	}
	if(remote_dev.num_dev==0){
		printf("No remote device found\ttry to scan\n");
		return;
	}
	printf("Connecting whit device %s to\n %s %s...\n",batostr(&(local_dev->info.bdaddr)),batostr(&(remote_dev.dev[dev].bdaddr)),remote_dev.name[dev] );
	/* connect to remote sdp running in remote device dev whit specified blutooth adapter */
	session = sdp_connect(&(local_dev->info.bdaddr),&(remote_dev.dev[dev].bdaddr),SDP_RETRY_IF_BUSY);
	if(session==NULL){
		perror("sdp connect");
		exit(-1);
	}

	result = sdp_service_search_attr_req(session,search,SDP_ATTR_REQ_RANGE,attr_range,&lst_response);
	if(result<0){
		/* an error */
		printf("Error while search service\n");
	}
	else{
/*	sdp_list_t *rsp_list
 *      This variable is set on a successful return to point to
 *      service(s) found. Each element of this list is of type
 *      sdp_record_t *.
*/
		char str[255],str2[255],desc_serv[255];
		uuid_t tmp_uuid;
		printf("Ok service found\n");

	        sdp_list_t *proto = NULL;
		int port;
		char proto_name[255];
	        sdp_list_t *r = lst_response;
		conn_num=0;
		for(;r;r=r->next){
			sdp_record_t *rec = (sdp_record_t*) r->data;
			sdp_uuid2strn(&(rec->svclass),str,255);
			sdp_get_service_name(rec,str2,255);
			printf("UUID %s %s\n",str,str2);

			conn_info[conn_num].handle = rec->handle;
			sprintf(conn_info[conn_num].desc,"UUID %s %s\n",str,str2);
			

			/* find proto and port or this service */
			if(sdp_get_access_protos(rec,&proto)<0){
				perror("No proto");
			}
			else{
				for(;proto;proto=proto->next){
					sdp_list_t *pds = (sdp_list_t*)proto->data;
					for(;pds;pds=pds->next){
						/* for each protocol list */
						sdp_data_t *data = (sdp_data_t*) pds->data;
						int p;
						for(;data;data=data->next){
							port = 0;
							sdp_proto_uuid2strn( &data->val.uuid ,proto_name,255);
							/* find service port available */
							p=sdp_uuid_to_proto(&data->val.uuid);
							port = sdp_get_proto_port(proto,p);
							if(port>0) conn_info[conn_num].port = port;
							printf("\tFind proto %s port %d\n ",proto_name,port);	
						}
					}
				}
				
			}
			sdp_record_free( rec );
			conn_num++;
		}
	}
	sdp_list_free( lst_response, 0 );
	sdp_list_free( search, 0 );
	sdp_list_free( attr_range, 0 );
	sdp_close( session );
	
	print_conn_info(conn_num);
}


