
/*
 * rfcomm_util.c - Utility functions for manipulate the rfcomm connection
 *
 * Copyright (C) 2012-2013  Davide Parise <bubini.mara5@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/

#include <stdio.h>

#include "h/rfcomm_util.h"

int local_socket;
/*
 * Connect to raddr device with rfcomm proto
 * param@laddr:: local device or BDADDR_ANY,LOCAL
 * param@raddr:: remote device address
 * param@port:: port to connct
 * return the socket
*/
int rfcomm_connect(bdaddr_t *laddr, bdaddr_t *raddr, uint port){
	int s,status;
	struct sockaddr_rc addr_remote = { 0 };
	struct sockaddr_rc addr_local = { 0 };
	
	if(raddr==NULL){
		printf("Error Select remote addr\n");
		return -1;
	}
	
	addr_local.rc_family = AF_BLUETOOTH;
	addr_local.rc_channel = 0;
	bacpy(&(addr_local.rc_bdaddr),laddr);

	addr_remote.rc_family = AF_BLUETOOTH;
	addr_remote.rc_channel = port;
	bacpy(&(addr_remote.rc_bdaddr),raddr);


	s = socket(AF_BLUETOOTH, SOCK_STREAM, BTPROTO_RFCOMM);
	if(s<0){
		perror("Connection sock()");
		return s;
	}
	printf("Sock ok\n");

	status = bind(s,(struct sockaddr *)&addr_local,sizeof(addr_local));
	if(status<0){
		perror("bind in connect");
		return status;
	}
	printf("Bind ok\n");


	status = connect(s, (struct sockaddr *)&addr_remote, sizeof(addr_remote));
	if( status < 0 ) {
		perror("Connect socket");
		return status;
	}
	printf("Connection Ok\n");
	local_socket = s;

	return s;
}

/* 
 * Void:accept connectio over rfcomm 
 * @Param:Void
 * @Return:
 * 	0 - if success
 *	-1 - otherwise
*/
int rfcomm_accept(){
	struct sockaddr_rc loc_addr = { 0 }, rem_addr = { 0 };
	char buf[1024] = { 0 };
	int s, client, bytes_read;
	socklen_t opt = sizeof(rem_addr);
	fd_set fds;
	int ret;

	s = socket(AF_BLUETOOTH, SOCK_STREAM | SOCK_CLOEXEC, BTPROTO_RFCOMM);

	loc_addr.rc_family = AF_BLUETOOTH;
	loc_addr.rc_bdaddr = *BDADDR_ANY;
	loc_addr.rc_channel = (uint8_t) 8;
	bind(s, (struct sockaddr *)&loc_addr, sizeof(loc_addr));


	listen(s, 1);

	/* select */
	FD_ZERO(&fds);
	FD_SET(s,&fds);
	ret = select(s+1, &fds, NULL, NULL, NULL);
	if (ret > 0 && FD_ISSET(s, &fds)) {
		/* select ok */
	}
	else{
		perror("rfcomm_accept:Select error\n");
		return -1;
	}
	
	// accept connection
	client = accept(s, (struct sockaddr *)&rem_addr, &opt);

	ba2str( &rem_addr.rc_bdaddr, buf );
	fprintf(stdout, "rfcomm:accepted connection from %s\n", buf);
	memset(buf, 0, sizeof(buf));

	local_socket = s;
	return client;

}
/*
 * close the opened socket 
 * param@sock:: the socket
*/
void rfcomm_close(int sock){
	close(sock);
	close(local_socket);
}
/* other like as listen, bind, create dev, dev info, ... */


