/*
 * systemControll_handler.c - Hanldler for shutdown and restart the system
 *
 * Copyright (C) 2012-2013  Davide Parise <bubini.mara5@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/

#include <stdio.h>

#include "h/msg.h"
#include "h/constant_msg.h"
#include "h/systemControll_handler.h"

#include "../lib/system/systemControll.h"

#define CMD_SHUTDOWN	0x01
#define CMD_LOGOUT	0x02
#define CMD_RESTART	0x03

msg_t err_system[] =	{ID_SYSTEM,FIELD_NOT_SET,TYPE_ERROR,FIELD_NOT_SET,0x00,0x00,0x00,0x00};
msg_t ack_system[] = 	{ID_SYSTEM,FIELD_NOT_SET,TYPE_ACK,  FIELD_NOT_SET,0x00,0x00,0x00,0x00};

void system_handler(){
	uint8_t typeL,typeM,*data;
	int coordX,coordY;
	uint32_t keyCode;
	int size;
	msg_t *msg;

	printf("******** SYSTEM HANDLER *******\n");
	
for(;;){

	if(read_msg(&msg)<0){
		fprintf(stderr,"Error system_handler: can not read messages\n");
		return;
	}

	if(getMsgTypeM(msg,&typeM)<0){
		fprintf(stderr,"Error system_handler: %s\n",strmsgerr(getmsgerr()));
		freeMsg(msg);
		return;
	}
	if(getMsgTypeL(msg,&typeL)<0){
		fprintf(stderr,"Error system_handler: %s\n",strmsgerr(getmsgerr()));
		freeMsg(msg);
		return;
	}

	/* check message change state */
	if(typeM & TYPE_STATE_CHANGE){
		switch(typeL){
			case STATE_START:	/* never here only in the server switcher */
				fprintf(stdout,"SYSTEM HANDLER:: not normal !! change state start\n");
				break;
			case STATE_STOP:
				printf("SYSTEM HANDLER:: command STOP\n");
				freeMsg(msg);
				if(send_msg(ack_system)<0)
					fprintf(stderr,"Error system_handler() State stop:can not send msg\n");
				return;
			case STATE_RESUME:	/* never here only in the server switcher */
				fprintf(stdout,"SYSTEM HANDLER:: not normal !! change state resume\n");
				break;
			default:
				printf("SYSTEM HANDLER:Change state:Unknow state 0x%.2x\n",typeL);
				if(send_msg(err_system)<0){
					fprintf(stderr,"Error system_handler() State default:can not send msg\n");
				}
		};
	}
	else if(typeM  == CMD_SHUTDOWN){
		if(system_shutdown()<0){
			send_msg(err_system);
		}

		else if(send_msg(ack_system)<0)
			fprintf(stderr,"Error system_handler() shutdown:can not send msg\n");
	}
	else if(typeM == CMD_RESTART){
		if(system_restart()<0){
			send_msg(err_system);
		}

		else if(send_msg(ack_system)<0)
			fprintf(stderr,"Error system_handler() restart:can not send msg\n");

	}
	else if(typeM == CMD_LOGOUT){
		system_logout();
		if(send_msg(ack_system)<0)
			fprintf(stderr,"Error system_handler() logout:can not send msg\n");
	}
	freeMsg(msg);
}/* END for(;;) */
}

